%affine_extras more affine model calculations
% Et(rx(n)_t+1 = gamma0_fit + rxloads*X;
% Ey1 = time series of expected one year yields.
% Ex = time series of expected x -- scaled now to give 10 year bond excess return

function [gamma0_fit, rx_loads, Ey1,fwd_decomp, Ex] = affine_extras(A,B,V,lam0,lam1,mu,phi,X)

T = size(X,1);
maxcalc = size(B,1);

gamma0_fit = zeros(maxcalc,1);
for i = 1:maxcalc;
    gamma0_fit(i) = B(i,:)*V*lam0 - 1/2*B(i,:)*V*B(i,:)'; % note gamma(1) is maturity (2)
end; 
rx_loads = B*V*lam1; 


% expected future interest rates -- starts at y(1)_t, E_ty(1)_t+1, etc. 
Ey1 = -(A(1)+X*B(1,:)') ;

% terms in decomposition of forward rates -- starts at f(2)_t
x = X(:,1);
fwd_decomp = zeros(T,maxcalc,maxcalc); % 1) when 2) which forward we're looking at 3) how far out to look (rxt_k)
Ext = zeros(T,maxcalc); % Et(x_t+j) T long by maxcalc wide 

fwd_decomp(:,:,1) =  ones(T,1)*(gamma0_fit' - [0; gamma0_fit(1:end-1,:)]')...
                     + x*(rx_loads(:,1)' - [0; rx_loads(1:end-1,1)]'); % E_t(rx^(n)_t+1)-E_t(rx^(n-1)_t+1)) 
Ex(:,1) = gamma0_fit(9) + rx_loads(9,1)*x;  % 10 year bond excess return
                 
EX = X;               
for i = 2:maxcalc
     EX = ones(T,1)*mu'+EX*phi';
     Ey1 = [Ey1  -(A(1)+EX*B(1,:)')];
     Ex(:,i) = gamma0_fit(9) + rx_loads(9,1)*EX(:,1);  
     
     fwd_decomp(:,:,i) = fwd_decomp(:,:,i-1) ...
                      + [zeros(T,i-1) ( ones(T,1)*(gamma0_fit(1:end-(i-1),:)' - [0; gamma0_fit(1:end-(i),:)]') ...
                      + EX(:,1)*(rx_loads(1:end-(i-1),1)' - [0; rx_loads(1:end-(i),1)]'))];  
                 % adds  E_t(rx^(n-j)_t+1+j)-E_t(rx^(n-j-1)_t+1+j))
end;

