% THIS IS NOW THE MAIN PROGRAM AND DOES EVERYTHING
% Many options of the old program are removed. Always mixes FB ad GSW data,
% always has MA right hand side, 4 factors, etc. 

clear all
close all

% --------------------------
% Options 
% ----------------------------

nolev = 1;  % no level information in expected returns. factor x is constructed using spreads, and model 
             % is constrained that only x may forecast returns. This is the only effect now -- imposing unit roots is gone
             % default is nolev = 1; haven't tried nolev = 0 in a while. 
             
do_intro = 0; % introductory analysis or skip to affine model. Just speeds up the program and cuts out lots of graphs

maxcalc = 20; % how far out to calculate term structure. Haven't tried changing it from 20 in a long time 

restrict_eigs = 0; % restrict eigenvalues to 0.999 in search for phistar. This does not limit eigenvalues of phi (until GMM or ML) 
                   % it's worth running both ways to see the differences. 

demean = 1; % option to specify the model in terms of de-meaned factors. Lets you control factor means. Reason: without,
            % the model is estimating wild factor means. mustar is poorly
            % identified, and so leads to huge errors in (I-phi)^-1 mu (mu
            % and mustar are about the same) ; the model thinks it's always
            % far from mean. Default is 1; without it you get weird means
            % and decompositions. 

            
% **************************
% GET DATA
% ************************


% -----------------------------
% load fed data 
% ----------------------------

load fed_yc_data

%Zero-coupon yield Continuously Comp. SVENYXX All integers 1-30
%Par yield Coupon-Equivalent SVENPYXX All integers1-30bac
%Instantaneous forward rate Continuously Comp. SVENFXX All integers 1-30
%One-year forward rate Coupon-Equivalent SVEN1FXX 1, 4, and 9
%Parameters N/A BETA0 to
%TAU2

% 7 years start in 1961. In 1971/11 we get maturities to 15 years. 

yd = [SVENY01 SVENY02 SVENY03 SVENY04 SVENY05 SVENY06 SVENY07 SVENY08 SVENY09 SVENY10...
    SVENY11 SVENY12 SVENY13 SVENY14 SVENY15 SVENY16 SVENY17 SVENY18 SVENY19 SVENY20...
    SVENY21 SVENY22 SVENY23 SVENY24 SVENY25 SVENY26 SVENY27 SVENY28 SVENY29 SVENY30]/100; 

plotdatesd = datenum(Year, Month, Day);

if 0; % just plots data.
    figure
    plot(plotdatesd,100*yd)
    datetick('x','yyyy')
    axis([datenum(1961,01,01) datenum(2006, 06, 01) -inf inf]);
end;

% convert to monthly data. Find index of end of month observations

indxmo = [];
Td = size(yd,1);
for t = 1:Td-1
    if ~(Month(t+1) == Month(t))
        indxmo = [indxmo; t]; 
    end;
end;  %indxmo now has indices of month-end observations

indxmo = indxmo(1:end-2); % chops last two months for compatibility with FB data below. 

y = yd(indxmo,:);
plotdates = plotdatesd(indxmo);
beta0 = beta0(indxmo);
beta1 = beta1(indxmo);
beta2 = beta2(indxmo);
beta3 = beta3(indxmo);
tau1 = tau1(indxmo); 
tau2 = tau2(indxmo); 
Year = Year(indxmo);
Month = Month(indxmo);
Day = Day(indxmo); 

T = size(y,1);

% chop to dataset with 15 years 

fd15 = find( isnan(y(1:end,15)) & ~isnan([y(2:end,15);0]))+1; % index of first date with 15 years of history

p = -(ones(T,1)*(1:30)).*y;

f = [y(:,1) p(:,1:29)-p(:,2:30)];
r = p(13:T,1:29)-p(1:T-12,2:30);
rx = r - y(1:T-12,1)*ones(1,29);
r = [NaN(12,1)*ones(1,29); r];     % pads out the initial values with zeros so same length as other series       
rx = [NaN(12,1)*ones(1,29); rx];   % NaN makes sure you don't inadvertently use them. 


y = y(fd15:end,1:15); 
rx = rx(fd15:end,1:14); 
f = f(fd15:end,1:15);
T = size(y,1);

beta0 = beta0(fd15:end);
beta1 = beta1(fd15:end);
beta2 = beta2(fd15:end);
beta3 = beta3(fd15:end);
tau1 = tau1(fd15:end); 
tau2 = tau2(fd15:end); 

Year = Year(fd15:end);
Month = Month(fd15:end);
Day = Day(fd15:end); 

% this figure shows why we don't use beta and tau as our factors, though they do describe the data perfectly
if 0;
    figure
    plot([beta0 beta1 beta2 beta3 tau1 tau2]);
    legend('beta0','beta1','beta2','beta3','tau1','tau2');
end;

% construct moving averages and adapt rest of data for moving averages

f_ma = (f(1:end-2,:)+f(2:end-1,:)+f(3:end,:))/3;
y_ma = (y(1:end-2,:)+y(2:end-1,:)+y(3:end,:))/3;

y = y(3:end,:);
rx = rx(3:end,:);
f = f(3:end,:);
T = size(y,1);
Year = Year(3:end);
Month = Month(3:end);
Day = Day(3:end);

maxmat = 15; % max maturity; 15 for GSW data, 5 for famabliss


dates = [find((Year==2001)&(Month==12));
         find((Year==2003)&(Month==12)); 
         find((Year==2006)&(Month==3))]; % indices of chosen dates in Fed calendar

     
% -------------------------------------
% Now load FB data. WIll merge the two datasets -- fb for return forecast factor, fed for other stuff
% -------------------------------------

            
x = load('fama_bliss_price.txt'); 
qdate = x(:,1); % e.g. 051230
prices = x(:,2:end); % percent prices, e.g. 98.234
clear x
T_fb = size(prices,1);
plotdate = (1952+06/12:1/12:1952+06/12+(T_fb-1)/12)'; % pretty dates for plots

beg = 140; %beginning date. 140 = 1964. Same as FB and previous data unreliable as per Fama. 
beg = 234; % 1971:11 date for merging with 15 year maturity fed data 
    beg = 234+2;  % fed data starts later if MA

qdate = qdate(beg:end); 
plotdate = plotdate(beg:end); 
prices_ma = (prices(beg-2:end-2,:)+prices(beg-1:end-1,:)+prices(beg:end,:))/3; 
prices = prices(beg:end,:); 
T_fb = size(prices,1); 

p_fb = log(prices/100); 
y_fb = -p_fb./(ones(T_fb,1)*[1 2 3 4 5]); 
f_fb = [y_fb(:,1) p_fb(:,1:4)-p_fb(:,2:5)];
r_fb = p_fb(13:T_fb,1:4)-p_fb(1:T_fb-12,2:5);
rx_fb = r_fb - y_fb(1:T_fb-12,1)*ones(1,4);
r_fb = [NaN(12,1)*ones(1,4); r_fb];     % pads out the initial values with zeros so same length as other series       
rx_fb = [NaN(12,1)*ones(1,4); rx_fb];   % NaN makes sure you don't inadvertently use them. 

p_ma_fb = log(prices_ma/100); 
y_ma_fb = -p_ma_fb./(ones(T_fb,1)*[1 2 3 4 5]); 
f_ma_fb = [y_ma_fb(:,1) p_ma_fb(:,1:4)-p_ma_fb(:,2:5)];

% ---------------
% load fed funds
% ------------------

x = load('fed_funds.txt'); 
ff = x(212:end-4,4); % 72:1 to 06:3 like FB data  

%dates of recent episodes

indx1 = find(qdate==900131);
indx2 = find(qdate==960329);
indx3 = find(qdate==000131); 


% ******************************************
% INTRODUCTORY ANALYSIS AND SPECIFICATION
% ******************************************

if do_intro; 
    dointro
end;

% **********************************
% FIT AFFINE MODEL
% ************************************

% -----------------------
% 1. Form factors 
% ---------------------------
% -----------------------------
% a. run forecasting regression; form return forecast factor
% -----------------------------

if nolev; 
        rhs = [ones(T-12*1,1) f_ma_fb(1:T-12*1,2:end)-f_ma_fb(1:T-12,1)*ones(1,4)];   
        bigrhs = [ones(T,1) f_ma_fb(1:T,2:end)-f_ma_fb(1:T,1)*ones(1,4)];
else;
        rhs = [ones(T-12*1,1) f_ma_fb(1:T-12*1,:)];   
        bigrhs = [ones(T,1) f_ma_fb(1:T,:)];
end;
     

lhs = rx(1*12+1:T,:);
[b1,stb,R2,R2adj,v,F] = olsgmm(lhs,rhs,12,0);
reterr = lhs-rhs*b1; % return errors saved for later

Erx = bigrhs*b1; 
alpha = b1(1,:)'; 
if nolev
    betas = [-sum(b1(2:end,:)); b1(2:end,:)]'; % express results in terms of regression on levles of f
else
    betas = b1(2:end,:)';
end; 

[Qr,Lr] = eig(cov(Erx)); 
Lrd = diag(Lr);

qr = Qr(:,end); 
qr2 = Qr(:,end-1);
qr3 = Qr(:,end-2); % saved for later

gammas = betas'*qr; % weights on forwards to construct rf factor

figure;
subplot(2,1,1)
plot((2:size(qr,1)+1)',qr,'v-')
title('qr, forms return forecast factor from expected returns'); 
subplot(2,1,2)
plot((1:5)',betas'*qr,'-v')
title('\gamma, forms return forecast factor from FB forwards');  
set(gca,'Xtick',1:5)

x = qr'*alpha + f_ma_fb*betas'*qr; % forms return-forecast factor -- changed so this IS the ER of qr'rx

% find intercepts to use with return forecast factor
% Et(rx(n)_t+1 is now gamma0(n-1) + qr(n-1)*x in single factor model
lhs = rx(12+1:T,:);
rhs = [ones(T-12,1)  x(1:T-12,:) ];
[b1,stb,R2,R2adj,v,F] = olsgmm(lhs,rhs,12,0);
gamma0 = b1(1,:)'; 

% with the new constant, gamma0 is very small -- we can impose zero, 
% extending the one factor model to the intercept too. 

% check return forecast factor is right
lhs = rx(12+1:T,:);
fprintf('regressing returns on gamma0 + qr*x\n'); 
fprintf('         %8s %8s \n', 'const(%)','x');
for i = 1:maxmat-1; 
    rhs = [ones(T-12,1)  gamma0(i)*ones(T-12,1)+qr(i)*x(1:T-12,:) ];
    [b1,stb,R2,R2adj,v,F] = olsgmm(lhs(:,i),rhs,12,0);
    fprintf('rx(%3i)  %8.2f %8.2f \n',i,[100 1]'.*b1);
end;
 
% -----------------------
% b. form other factors
% ----------------------

lhs = f; 
rhs = [ones(T,1)  x ];
[b1,stb,R2,R2adj,v,F] = olsgmm(lhs,rhs,12,0);
a = b1(1,:)';  
b = b1(2,:)';
e = lhs-rhs*b1; 
[Q,L] = eig(cov(e));
if ~min(diag(L) == sort(diag(L)));
      disp('**************ERROR eigenvalues are in wrong order'); 
end;

ql = Q(:,end); 
qs = -Q(:,end-1);% so that + slope means low short rates and high long rates -- Q gave the opposite sign 
qc = Q(:,end-2); 

level = (ones(T,1)*a'+e)*ql; % forms other factors
slope = (ones(T,1)*a'+e)*qs; 
curve = (ones(T,1)*a'+e)*qc;
    
X = [x level slope curve];  % <<<<<<<<<<< Here is the main definition of the factors  
    
 % report fractions of variance. standardize first factor so sum of squared loadings = 1

disp(' '); 
fprintf(' standard deviation of factors and fractions of variance\n'); 
fprintf('           %8s %8s %8s %8s \n', 'x','level','slope','curve')
fprintf(' std dev   %8.2f %8.2f %8.2f %8.2f \n', 100*std(x)*sum(b.^2).^0.5,100*std(level),...
           100*std(slope),100*std(curve))
totalvar = sum(diag(cov(lhs))); 
fprintf(' pct var   %8.2f %8.2f %8.2f %8.2f \n', 100*var(x)*sum(b.^2)/totalvar,...
        100*var(level)/totalvar,100*var(slope)/totalvar,100*var(curve)/totalvar)
   
% form demeaned factors for that option

if demean
    meanX = mean(X)';
    X = X-ones(size(X,1),1)*meanX';
end;

% ---------------------------------------------------------
% c. Loadings of forward rates on factors in statistical model
%  this is a benchmark for cross-sectional fitting; you can't do better
%  than OLS (at least by MSE criterion).  The a,b are also needed as
%  starting values for delta0 and delta1 in minimization below. Don't
%  overwrite them!
% -----------------------------------------------------------

rhs = [ones(T,1) X];
[b1,stb,R2,R2adj,v,F] = olsgmm(f,rhs,0,0);
a = b1(1,:)';
b = b1(2:end,:)';  % <<<<<<<<<<<< these get used below, don't screw them up! 
ffit_stat = rhs*b1;
    
%    corr(f-ffit_stat); 
%see this to understand GLS weighting matrix in minimization below. 

% ------------------------
% d. Check, nothing but return forecast factor forecasts returns
% yes, passes the test for level specification
% for nolev specification, level does seem to forecast -- of course
% -------------------------------

 lhs = rx(12+1:T,:);
 rhs = [ones(T-12,1)  X(1:T-12,:) ];
 [b1,stb,R2,R2adj,v,F] = olsgmm(lhs,rhs,12,0);
 
 disp(' ');
 fprintf('regressing returns on factors\n');
 fprintf('         %8s %8s %8s %8s %8s\n', 'const','x','level','slope','curve','R2');
 fprintf('coeffs \n');
 fprintf('rx(2)    %8.2f %8.2f %8.2f %8.2f %8.2f\n',b1(:,1));
 fprintf('rx(3)    %8.2f %8.2f %8.2f %8.2f %8.2f\n',b1(:,2));
 fprintf('rx(4)    %8.2f %8.2f %8.2f %8.2f %8.2f\n',b1(:,3));
 fprintf('rx(5)    %8.2f %8.2f %8.2f %8.2f %8.2f\n',b1(:,4));
 fprintf('t stats, R2 \n');
 fprintf('rx(2)    %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',b1(:,1)./stb(:,1),R2(1));
 fprintf('rx(3)    %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',b1(:,2)./stb(:,1),R2(2));
 fprintf('rx(4)    %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',b1(:,3)./stb(:,1),R2(3));
 fprintf('rx(5)    %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',b1(:,4)./stb(:,1),R2(4));

 lhs = rx(12+1:T,:);
 rhs = [ones(T-12,1)  X(1:T-12,[1 3 4]) ];
 [b1,stb,R2,R2adj,v,F] = olsgmm(lhs,rhs,12,0);

 disp(' ');
 fprintf('regressing returns on factors omitting level\n');
 fprintf('         %8s %8s %8s %8s %8s\n', 'const','x','level','slope','curve','R2');
 fprintf('coeffs \n');
 fprintf('rx(2)    %8.2f %8.2f %8.2s %8.2f %8.2f\n',b1(1:2,1),' ',b1(3:end,1));
 fprintf('rx(3)    %8.2f %8.2f %8.2s %8.2f %8.2f\n',b1(1:2,2),' ',b1(3:end,2));
 fprintf('rx(4)    %8.2f %8.2f %8.2s %8.2f %8.2f\n',b1(1:2,3),' ',b1(3:end,3));
 fprintf('rx(5)    %8.2f %8.2f %8.2s %8.2f %8.2f\n',b1(1:2,4),' ',b1(3:end,4));
 fprintf('t stats, R2 \n');
 fprintf('rx(2)    %8.2f %8.2f %8.2s %8.2f %8.2f %8.2f\n',b1(1:2,1)./stb(1:2,1),' ',b1(3:end,1)./stb(3:end,1),R2(1));
 fprintf('rx(3)    %8.2f %8.2f %8.2s %8.2f %8.2f %8.2f\n',b1(1:2,2)./stb(1:2,2),' ',b1(3:end,2)./stb(3:end,2),R2(2));
 fprintf('rx(4)    %8.2f %8.2f %8.2s %8.2f %8.2f %8.2f\n',b1(1:2,3)./stb(1:2,3),' ',b1(3:end,3)./stb(3:end,3),R2(3));
 fprintf('rx(5)    %8.2f %8.2f %8.2s %8.2f %8.2f %8.2f\n',b1(1:2,4)./stb(1:2,4),' ',b1(3:end,4)./stb(3:end,4),R2(4));


 lhs = rx(12+1:T,:);
 rhs = [ones(T-12,1)  X(1:T-12,1) ];
 [b1,stb,R2,R2adj,v,F] = olsgmm(lhs,rhs,12,0);

 disp(' ');
 fprintf('regressing returns on x factors only\n');
 fprintf('         %8s %8s %8s %8s %8s\n', 'const','x','level','slope','curve','R2');
 fprintf('coeffs \n');
 fprintf('rx(2)    %8.2f %8.2f \n',b1(:,1));
 fprintf('rx(3)    %8.2f %8.2f \n',b1(:,2));
 fprintf('rx(4)    %8.2f %8.2f \n',b1(:,3));
 fprintf('rx(5)    %8.2f %8.2f \n',b1(:,4));
 fprintf('t stats, R2 \n');
 fprintf('rx(2)    %8.2f %8.2f %8.2f \n',b1(:,1)./stb(:,1),R2(1));
 fprintf('rx(3)    %8.2f %8.2f %8.2f \n',b1(:,2)./stb(:,1),R2(2));
 fprintf('rx(4)    %8.2f %8.2f %8.2f \n',b1(:,3)./stb(:,1),R2(3));
 fprintf('rx(5)    %8.2f %8.2f %8.2f \n',b1(:,4)./stb(:,1),R2(4));


 disp(' SHOW THIS -- its why we need an affine model')

% ---------------------------
% 2. Unconstrained VAR estimate of factor dynamics 
% X_t+1 = mu_ols + phi_ol X_t + v_t+1; V = E(vv'); 
% form V matrix used througout below 
% -----------------------------

lhs = [X(1+12:T,:)]; 
rhs = [ones(T-12,1) X(1:T-12,:)]; 
[b1,stb,R2,R2adj,v,F] = olsgmm(lhs,rhs,12,0);
    
 % 4 factor model
disp(' '); 
fprintf('Raw estimate of three-factor VAR at annual horizon\n'); 
fprintf('         %8s %8s %8s %8s %8s %8s \n','const','x','level','slope','curve','R2'); 
fprintf(' x       %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',100*b1(1,1),b1(2:end,1),R2(1));
fprintf(' level   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',100*b1(1,2),b1(2:end,2),R2(2));
fprintf(' slope   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',100*b1(1,3),b1(2:end,3),R2(3));
fprintf(' curve   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',100*b1(1,4),b1(2:end,4),R2(4));
fprintf('tstats\n');
fprintf(' x       %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',b1(:,1)./stb(:,1),R2(1));
fprintf(' level   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',b1(:,2)./stb(:,2),R2(2));
fprintf(' slope   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',b1(:,3)./stb(:,3),R2(3));
fprintf(' curve   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',b1(:,4)./stb(:,4),R2(4));
     
mu_ols = b1(1,:)';  % <<<<<<<<<<<<<<< These get used throughout below
phi_ols = b1(2:end,:)'; % direct estimates of VAR

err = lhs-rhs*b1;
V = cov(err); % <<<<<<<<<<<< IMPORTANT this V matrix gets used in everything that follows
disp('correlation matrix of VAR errors')
disp(corr(err))

    
% *********************************
% 3. Explorations of market prices of risk
% this section establishes the two-number structure of the market prices of
% risk, and then finds the exact-identified values 
% to nail the regression of qr'rx on x, and the mean interest rate. 
% 
% while conceptually this comes after finding risk neutral dynamics, 
% the minimizer for the latter needs a good starting value. phi = eye(4)
% does not work for example. phistar = OLS phi + V*lambda does work as a
% starting value. SO we have to find these simple lambdas first. 
% *************************************

% -------------------------------
% a. find lambdas - prices of risk -- by lining up mean returns with
% covariances. 
% lesson -- price of risk is almost entirely price of level shocks. 
% -------------------------------

C = reterr'*err/(T-12); % cov_t rx_t+1, v_t+1 of return and VAR shocks

lam1 = inv(C'*C)*C'*qr; % lam1 is the cross sectional regression
fit1 = C*lam1; 
lam1a = C(:,2)\qr; % lam1a is a cross sectional regression, imposing zero price of risk for other than level
fit2 = C(:,2)*lam1a; 

disp('lambda 1 -- loadings on all three'); 
fprintf( ' %8s %8s %8s %8s\n',  'x','level','slope','curve')
fprintf( ' %8.2f %8.2f %8.2f %8.2f \n',lam1);
disp('lambda 1 if restricted to load only on level');
fprintf( ' %8.2f %8.2f %8.2f %8.2f \n', 0,lam1a,0,0);

% plot to argue for one market price of risk


sv = std(err); 
if nolev;
    scale = [ 50 -6 15 -20 ] ;
end;
if ~nolev
     scale = [ 70 -5 -20 -40 ] ;
end;
figure;  
plot((2:size(qr,1)+1)',fit2,'--om','LineWidth',2); % fit first, so will be below
hold on
%plot((2:size(qr,1)+1)',fit1,'--ob','LineWidth',2);
plot((2:size(qr,1)+1)',qr,'-sk','LineWidth',2,'MarkerFaceColor','k')
text(size(qr,1)+1.3,qr(end),'q_r','Color','k'); 
hold on
plot((2:size(qr,1)+1)',[scale(1)*C(:,1)/sv(1) scale(2)*C(:,2)/sv(2) scale(3)*C(:,3)/sv(3) scale(4)*C(:,4)/sv(4)],'--v','LineWidth',2)
colors = get(gca,'ColorOrder');
text(size(qr,1)+1.3,scale(1)*C(end,1)/sv(1),'cov(r,x)','Color',colors(1,:));
text(size(qr,1)+1.3,scale(2)*C(end,2)/sv(2),'cov(r,level)','Color',colors(2,:));
text(size(qr,1)+1.3,scale(3)*C(end,3)/sv(3),'cov(r,slope)','Color',colors(3,:));
text(size(qr,1)+1.3,scale(4)*C(end,4)/sv(4),'cov(r,curve)','Color',colors(4,:));

%legend('Er = qr','cov(r,x)','cov(r,l)','cov(r,s)','cov(r,c)','all loading fit','level only fit',2)

set(gca,'Xtick',[(2:size(qr,1)+1)])
set(gca,'Ytick',[]); 
xlabel('Maturity, years')

if nolev
    axis([1 18 -0.1 0.8]); 
end;
if ~nolev;
    axis([1 18 -0.1 0.6]); 
end

if ~nolev
    print -depsc2 factor_fit_fed_4f.eps
else
    print -depsc2 factor_fit_nolev_fed.eps
end

 % same for lambda 0 
 
lam0 = inv(C'*C)*C'*(gamma0 + 1/2*var(reterr)');
fit1 = C*lam0; 
disp('lambda 0 -- loadings on all factors'); 
disp(lam0'); 

lam0a = C(:,2)\(gamma0 + 1/2*var(reterr)'); 
disp(' loading if only level') 
disp(lam0a)
fit2 = C(:,2)*lam0a;

figure; 
plot(10*C,'-v')
hold on
plot((gamma0 + 1/2*var(reterr)'),'-or')
plot(fit1,'--+b'); 
plot(fit2,'--+g')
legend('x','l','s','\gamma_0 + 1/2 \sigma^2','all loading fit','level only fit')

clear lam0 lam1 % these are reserved for matrix size items from here on

% ---------------------
% this section nends with lam0a and lam1a estimated from cross sectional
% regressions imposing only level shocks have market prices of risk. 
% they are numbers, pad them out to matrices before using in eg phi* = phi
% - lambda V
% -----------------------------

% *******************************************
% 4. Complete OLS based models by subtracting market price of risk correction from
% the OLS meausre of true-probability dynamics. This defines the "OLS"
% estimate of the affine model (fit REAL dynamics, then find implied risk nuetral dynamics) 
% and also is the starting point for fitting the factors. 
% *********************************************

lam0 = [ 0 lam0a 0 0]' ; 
lam1 = [ [ 0 lam1a 0 0]' zeros(4,3)]; % pad out to usual matrix size
mustar_ols = mu_ols - V*lam0;   
phistar_ols = phi_ols - V*lam1;

delta0_ols = a(1);  % The "OLS model" also uses these regression loadings for y(1)
delta1_ols = b(1,:)'; 

 show_results(delta0_ols,delta1_ols,[lam0a lam1a]',phistar_ols,mustar_ols,X,V,f,f_ma_fb,'OLS-Affine',dates,qdate,plotdate,indx3);

% *********************************
% 4. Minimization of fitting errors to estimate phi
% *********************************

% -----------------------------------
% a. Now, search for affine model risk neutral dynamics to best match
% forwards. Market prices of risk do not enter here. We needed them only to
% define decent starting values for the search. 

% general approach: search over riskneutral dynamics mustar, phistar,
% delta0 and delta1 to best fit the cross section of forward rates. This is
% equivalent to best fitting the OLS loadings of forwards on factors, i.e.
% f = Af + BfX comes closest to f is the same as coming closest to
% ffit_stat = A + BX + e
% --------------------------------------------

% -------------------------------------
% there is an unused option here to use
% weights, like ML can weight across maturities. 
% This is potentially important. iid across maturities is a very bad model. the errors (see errors in ffit_stat) are 
% of coruse highly correlated across maturity, since the "data" is a 6 factor model. In practical terms, iid means 
% you can get negative eigenvalues and sawtooth loadings. You'd rather larger departures but smooth across maturities. 
% in turn that preference comes from understanding that fitting errors are correlated across maturities so you should have something 
% like an AR(1) correction. ML below implements this. However, in the end the fit is so good here it was not necessary. 
% Any standard errors or statistical treatment of this minimization should
% incorporate the huge cross sectional correlation of fitting errors
% however. 
% ------------------------------------

if 1 ; 
     wts = 1; % codes for don't use any weights
end;
if 0; 
     wts = eye(15); %-- lets you check compared to wts == 1 -- should give same answer 
end;
% each of these ideas needs some refining -- they are not checked to work
% with the final specification. ALso see the quasi first difference in
% minobj. 
if 0;
    wts = eye(15); 
    for i = 2:14; 
        wts(i,i+1) = -.7; 
    end;
    %wts(15,15)=0; -- this exactly gives the quasi first difference in the
    %program; 
    wts = wts'*wts; %weighting matrix corresponding to .7 ar(1) correction
end;
if 0;
    [Q D] = eig(cov(f-ffit_stat));
    num_eig = 1; % number of eigenvalues to include
    wts = Q(:,end-num_eig+1:end)*inv(D(end-num_eig+1:end,end-num_eig+1:end))*Q(:,end-num_eig+1:end)';   
    wts = inv(diag(diag(cov(f-ffit_stat)))); % start with iid across N
end;

% ----------------------------
% Here's the minimization 
% -------------------------

options = optimset('MaxFunEvals',20000,'Maxiter',20000);
disp('initial objective -- using OLS mu, phi and delta = a/b')

if ~demean
    initial = [ mustar_ols; reshape(phistar_ols,16,1); a(1); b(1,:)'];
    % there is an option here to search over some parameters fixing others.
    % It's historical from my efforts to make bad models fit better. Final
    % results don't use it, but since the future might need it, it's still
    % here
    %          mustar         phistar                        d0     d1  
    which = [ 1 1 1 1     1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1    1    1 1 1 1]'; % chooses which parameters to search over 
end
if demean
   % demean option forces mu=0000. we search only over lambda0
   initial = [ -10; reshape(phistar_ols,16,1); a(1); b(1,:)'];
    %        lambda0l      phistar                    d0     d1  
    which = [ 1    1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1    1    1 1 1 1]'; % chooses which parameters to search over 
end;

subs = initial; % values to use for non-search variables
initial = initial(which==1); % compact to only search over ones you want
min_options = [restrict_eigs demean]';
disp(minobj(initial,V,X,f,wts,min_options,which,subs));

% What's up with all the searches? fminunc by itself gets stuck ("cant find
% an acceptable point along the current search direction"). fminsearch
% doesnt' get close enough. So first fminsearch to provide decent starting
% values for fminunc. Then fminunc terminates with "gradient less than
% options.tol.fun" The restrict-eigs fminunc is still terminating with a
% line search criterion -- needs some fussing to get to the absolute best.
% However, fit seems good enough for now. 
% Big point: CHECK HOW MINIMIZATION ENDS. These models can all get around
% 10 bp of error. 

% Currently we're still not getting close enough. I have seen 10 bp errors,
% and 4 bp over the OLS model. and this is leaving 16 bp errors. This is
% just a question of good starting values and organizing the search better.
% 

min_options = [0 demean]';
disp('Starting minsearch');
[finalpars,finalobj] = fminsearch(@(minpars) minobj(minpars,V,X,f,wts,min_options,which,subs),initial,options);
disp('Starting minunc');
[finalpars,finalobj,exitflag,output,grad] = fminunc(@(minpars) minobj(minpars,V,X,f,wts,min_options,which,subs),finalpars,options);

if restrict_eigs
    min_options = [restrict_eigs demean]';
    disp('Starting minsearch with restricted eigenvalues');
    [finalpars,finalobj] = fminsearch(@(minpars) minobj(minpars,V,X,f,wts,min_options,which,subs),finalpars,options);
    disp('Starting minunc');
    [finalpars,finalobj,exitflag,output,grad] = fminunc(@(minpars) minobj(minpars,V,X,f,wts,min_options,which,subs),finalpars,options);
end

% Always check results with the plot of loadings vs.
% regression -- the result should be visually indistinguishable
% 
% Note: much better (still not great) convergence with demean option. 

disp('objective after searcch')
disp(finalobj);
[ mustar phistar delta0 delta1] = decode_level(finalpars,which,subs,demean);
if demean
    lam0l = mustar;
    mustar = -V(:,2)*mustar; % mustar was really lambda0l
    disp('demean option. Lambda0l and mustar are:') 
    disp(lam0l)
    disp(mustar');
end;

% -----------------------------------------------------------
% we are done with risk neutral dynamics -- show results. 
% -----------------------------------------------------------

lambdas = [0 0]';
[A,B,Af,Bf,mu,phi,lam0,lam1,gamma0_fit,rx_loads,Ey1,fwd_decomp,Ext] = ...
    show_results(delta0,delta1,lambdas,phistar,mustar,X,V,f,f_ma_fb,'Risk-neutral',dates,qdate,plotdate,indx3);

% -------------------------------------------------------------
% b. Now, some slightly better  market prices of risk
% 
% rather than cross sectional regression, 
% we can pick lambda0 and lambda 1 to 
% exactly match the return forecast regression
% qr' rx_t+1 = 0 + 1*x_t
% this produces the lam0b and lam1b estimates
% in the end they are virtually the same as cross sectional regression
% but much prettier to describe. 
% with the demean option, we keep lam0l as estimated above. It makes a tiny
% difference, but means that mu=0 exactly,which we want to keep control of
% E(X) 
% -----------------------------------------------------------------

vec1 = zeros(14,1);
vec2 = zeros(14,1);

for i = 1:14; 
    vec1(i) = B(i,:)*V*B(i,:)';
    vec2(i) = B(i,:)*V(:,2);
end; 

lam0b = 1/2*qr'*vec1/(qr'*vec2);
lam1b = 1/(qr'*vec2);                  % these values exactly match the regression forecast of qr'rx

disp('lambda matching only regression forecast x') 
disp([lam0b lam1b]); 
disp('old lambda from cross sectional regression') 
disp([lam0a lam1a]); 

% lam0c is a value that hits the SLOPE coefficient, but abandons
% the intercept in order to exactly nail the unconditional average yield(1)
% I did this before the demain option because the other estimates were leading to quite high average
% yields -- I wanted to see if a small change in intercept and lam0 could
% give us a more reasonable Ey. Yes, of course. Alas, the implied E(x) is
% really affected in this experiment. As documented a lot more in the GMM
% stuff, only lam0 has any effect on means. YOu can force one factor or
% linear combination to hit its mean, but the others will then be further
% off. 

% this is all historical -- not used with the demean option, but may prove
% useful later

phi = phistar + V*[[0 lam1b 0 0]' zeros(4,3)];
lam0c = (mean(X)*delta1 - delta1'*inv(eye(4) - phi)*mustar)/(delta1'*inv(eye(4)-phi)*V(:,2)); 
 % matching delta0 + delta1'X vs. matching y1 itself makes a very small difference

disp('lambda to match slope on x, abandon constant on x, match Ey(1)')
disp( [lam0c lam1b]); 
     
% *************************************************
% show results using these market prices of risk 
% ************************************************

if ~demean
    lambdas = [lam0b lam1b]';
   % mustar_new = (eye(4)-phi)*(mean(X))'; 
        % mustar that delivers mean X for sure. this is an option I used to
        % force decent means on X, pointing the way to the demean option
end
if demean
    lambdas = [lam0l lam1b]'; % use the value above from search. This should generate mu = 0 exactly. 
end

graph_label = 'Return-forecast';
[A,B,Af,Bf,mu,phi,lam0,lam1,gamma0_fit,rx_loads,Ey1,fwd_decomp,Ext] = ...
    show_results(delta0,delta1,lambdas,phistar,mustar,X,V,f,f_ma_fb,graph_label,dates,qdate,plotdate,indx3);

show_results(delta0,delta1,[lam0l lam1b/2]',phistar,mustar,X,V,f,f_ma_fb,'half-lambda',dates,qdate,plotdate,indx3);

% plot of eigenvalues. Nothing to do about eigenvalue larger than 1. 
fracl = (0:0.02:1.3)';
for i =1:size(fracl,1)'; 
    phit = phistar + V*lam1*fracl(i);
    eigvec(i,:) = sort(abs(eig(phit)));
end;
figure;
plot(lam1b*fracl,eigvec); 
hold on
plot(lam1b*[1 1]',[0 1.1]', '-r');
plot(lam1b*fracl,1+0*fracl,':k'); 
xlabel('Market price of risk \lambda_1')
ylabel('Eigenvalues of \phi')

   


if ~demean
    % this is the version with lambda0 chosen to nail the long term mean yield.
    % It does, but notice E(x) now explodes. One parameter can't control all
    % four means. Demean is a much cleaner solution to the problem 
    show_results(delta0,delta1,[lam0c lam1b]',phistar,mustar,X,V,f,'Mean-yield',dates,qdate,plotdate,indx3);
end; 

STOP

% *************************************************************************
% *************************************************************************
% Everything below here is historical. 
% 
% 1. The GMM estimate was an attempt to control out-of-control factor means by
% trading off the regression moment estimates of market prices of risk with
% moments that define means of the factors. Since it only had one parameter
% to control means with (lambda1 turned out to be the only important
% parameter) it wasn't able to control mean level and mean x at the same
% time. Nice level meant large unconditional risk premia and vice versa. 
% The demean option takes care of all this. 

% However, this may be a good starting point for a statistical
% investigation. It remains true that almost all sampling error comes from
% the unconditional means of the factors and the two market prices of risk.
% Casting the above as GMM may give us a simple distribution theory. 

% 2. After GMM comes a ML estimate which included the likelyhood of the
% risk-neutral fitting procedure as well as the market price of risk
% estimates. The likelyhhood was formed from cross sectional errors f = Af
% - Bf*X as well as time series errors X_t - mu -phi X_t-1. The idea here
% was that by allowing slightly worse cross sectional fit, you could
% slighty alter the huge eigenvalues in phistar and get much better time
% series fit. It didn't work for that end. Again, though, this could be the
% framework for doing ML, if that remains interesting (Do we want to trade
% off cross sectional fit for something better in the time series?). 

% The function calls have not been updated, so it won't run as is. Neither
% has been updated to reflect the demeaned factors. (Or, better, treating
% factor means EX as parameters in place of mu) 

% 3. There is a much longer set of graphs. I pulled the most interesting
% ones into show_results, and don't use these anymore. They also don't run
% as-is since they requires Af Bf, etc. from whichever "final" model you
% want graphs. (I did show_results so we could do the same graphs from many
% models.) 


if 1; % GMM to choose lambdas, balancing expected returns and unconditional means 
    % returns gmm moments. first is constant, next 4 are regression
    % orthogonality to x, level, slope, curve, next 4 are means of X
    % like everything else, V is fixed but should eventually update
    
    % check that lambdas engineered to nail regression deliver the right
    % GMM moments
    lambdas = [lam0b lam1b]';
    disp('lambdas picked to nail regression at 0 1'); 
    disp(lambdas')
    disp('moments at these lambdas - first two should be zero if you start with lamb0b lam1b'); 
    disp(findg(lambdas,phistar, mustar,delta0, delta1,qr,A,B,X,V,rx)');
                    % const x l s c Ex El Es Ec
    wts = diag([  1    1 0 0 0  0  0  0 0]');

    [lambdasfin,S,d,lam0,lam1,mu,phi] = dogmm(lambdas,1,'regression',phistar,mustar,delta0,delta1,qr,A,B,X,V,rx,wts);

    % check that lambdas engineered to nail E(y1) deliver the right moments
    lambdas = [lam0c lam1b]';
    disp(' ');
    disp('lambdas picked to nail regression 1 and E(y)'); 
    disp(lambdas')
    disp('moments at these lambdas - '); 
    oldg =   findg(lambdas,phistar, mustar,delta0, delta1,qr,A,B,X,V,rx);
    disp(100*oldg');
    lam0 = [ 0 lambdas(1) 0 0]'; 
    lam1 = [ [ 0 lambdas(2) 0 0]' zeros(4,3)]; % pad out to usual matrix size
    phi = phistar + V*lam1;
    mu = mustar + V*lam0;
    BVB = zeros(14,1);
    for i = 1:14; 
        BVB(i) = B(i,:)*V*B(i,:)';% BVB(1) corresponds to rx(2)
    end;
   
    predcon = -1/2*qr'*BVB + qr'*B(1:14,:)*V(:,2)*lambdas(1); 
    predslope = qr'*B(1:14,:)*V(:,2)*lambdas(2); 
    predmeanx =  inv(eye(4)-phi)*mu; 
    fprintf('predicted constant and slope   %8.3f %8.3f \n',predcon,predslope); 
    fprintf('predicted mean of factors, y1  %8.3f %8.3f %8.3f %8.3f %8.3f\n',predmeanx,100*(delta0+delta1'*predmeanx)); 
    fprintf('  sample  mean of factors, y1  %8.3f %8.3f %8.3f %8.3f %8.3f\n',mean(X),100*(delta0+mean(X)*delta1)); 

    % nailing Ey(1) by GMM 
    disp(' '); 
    lambdas = [lam0c lam1b]';
    disp('GMM minimization to nail EY(1)'); 
                % const x l s c Ey 
    wts = diag([  0    0 0 0 0  1  ]');
    disp('initial GMM objective')
    disp(gmmobj_y1(lambdas,phistar, mustar,delta0, delta1,qr,A,B,X,V,rx,wts))
    [lambdas, gvalue, exitflag, output] =...
        fminunc(@(lambdas) gmmobj_y1(lambdas,phistar, mustar,delta0, delta1,qr,A,B,X,V,rx,wts),lambdas); 
    disp('final GMM objective')
    disp(gvalue)          
    disp('new lambdas'); 
    disp(lambdas'); 
    disp('gT'); 
    newg = findg(lambdas,phistar, mustar,delta0, delta1,qr,A,B,X,V,rx);
    disp(100*newg');
    lam0f = lambdas(1); 
    lam1f = lambdas(2); 
    lam0 = [ 0 lam0f 0 0]'; 
    lam1 = [ [ 0 lam1f 0 0]' zeros(4,3)]; % pad out to usual matrix size
    phi = phistar + V*lam1;
    mu = mustar + V*lam0;    
    predcon = -1/2*qr'*BVB + qr'*B(1:14,:)*V(:,2)*lam0f; 
    predslope = qr'*B(1:14,:)*V(:,2)*lam1f; 
    predmeanx =  inv(eye(4)-phi)*mu; 
    fprintf('predicted constant and slope (0,1?) %8.3f %8.3f \n',predcon,predslope); 
    fprintf('predicted mean of factors, y1  %8.3f %8.3f %8.3f %8.3f %8.3f\n',predmeanx,100*(delta0+delta1'*predmeanx)); 
    fprintf('  sample  mean of factors, y1  %8.3f %8.3f %8.3f %8.3f %8.3f\n',mean(X),100*mean(f(:,1))); 

    % The major problems with the above is that by nailing Ey(1) we lose
    % control of E(x). (Notice the "predicted E(x) rows). Here I try to get
    % both E(x) and E(l). This is part of the non-identification from means in the paper -- means do not separately identify lo and l1 

    show_results(delta0,delta1,lambdas,phistar,mustar,X,V,f,'Mean-Y',dates,qdate,plotdate,indx3)
    
    % Here you see the problem -- try to hit two means with two lambas -- no can do  
    disp(' '); 
    lambdas = [lam0c lam1b]';
    disp('GMM minimization to nail means'); 
                % const x l s c Ex El Es Ec
    wts = diag([  0    0 0 0 0  1  1  0 0]');
    [lambdas,S,d,lam0,lam1,mu,phi] = dogmm(lambdas,1,'E(x), E(l)',phistar,mustar,delta0,delta1,qr,A,B,X,V,rx,wts);
    
    
    % ignoring that problem, let's go on and do real GMM. The objective is
    % to trade hitting the means of hte factors for the regression
    % coefficients
    disp(' '); 
    lambdas = [lam0c lam1b]';
    disp('GMM minimization to balance regressions and means'); 
                % const x l s c Ex El Es Ec
    wts = diag([  1    1 0 0 0  1  1  0 0]');
    [lambdas,S,d,lam0,lam1,mu,phi] = dogmm(lambdas,1,'regression and Ex,El, 1st stage',phistar,mustar,delta0,delta1,qr,A,B,X,V,rx,wts);
          
    
    % second stage. 
    
    % The program allows lots of moments. We only want to pay
    % attention to constant, e*x, E(x) and E(level). Rather than rewrite
    % the program, we trick it by using a S^-1 matrix that has a bunch of
    % zeros in it corresponding to moments we want to ignore

    disp(' '); 
    disp('Second stage GMM minimization to balance regressions and means');               
    wts2 = pinv(wts*S*wts');
    [lambdas,S,d,lam0,lam1,mu,phi] = dogmm(lambdas,1,'regression and Ex,El, 2nd stage',phistar,mustar,delta0,delta1,qr,A,B,X,V,rx,wts2);

    % calculate results for this model
    show_results(delta0,delta1,lambdas,phistar,mustar,X,V,f,'GMM2',dates,qdate,plotdate,indx3)

    % sensitivity of maximum eigenvalue to lambda2
    
    test = (-180:1:0)';
    for i = 1:size(test,1);
          lam1t = [ [ 0 test(i) 0 0]' zeros(4,3)]; % pad out to usual matrix size
           phit = phistar + V*lam1t;
          maxlam(i) = max(abs(eig(phit)));         
          lamst(i,:) = sort(abs(eig(phit)));
    end

    figure
    plot(test,lamst)
    hold on
    plot(lambdas(2),max(abs(eig(phi))),'vr')
    title('eigenvalues as function of \lambda_1');
    xlabel('\lambda_1')
    ylabel('abs eig (\phi)'); 
   
    
end; 
stop

mustar_1 = mustar;
phistar_1 = phistar; 
Af_1 = Af;
Bf_1 = Bf; 
V1 = V; 
phi_1 = phi;
mu_1 = mu; % save these for comparison graphs later
lam0_1 = lam0; 
lam1_1 = lam1; 
delta0_1 = delta0; 
delta1_1 = delta1; 

% this really just shows how darn close you can get with the affine model
% if you're willing to accept eigenvalues greater than one

% --------------------------------
% Now, once again. We need to restrict this to produce eigenvalues less
% than one as starting point for ML. If you give ML an initial value with
% eigenvalues greater than one, it can't find it's way back to realit
% ------------------------------------------------------
if 0; 
wts = 1; 
options = optimset('MaxFunEvals',20000,'Maxiter',20000);
%optim_method = 'loadings'; % choose 'forwards' or 'loadings'. 
optim_method = 'forwards'; % choose 'forwards' or 'loadings' or 'fit_fwds'. 
search_delta = 1; 
restrict_eigenvalues = 1; 
    % in minobj you can choose to min sum (f-ffit)^2 or to match the ols loadings. It gives the same answer. 
   
disp('Optimization method')
disp(optim_method);
disp('initial objective')
    if search_delta;
        initial = [ mustar; reshape(phistar,size(phistar,1)^2,1); a(1); b(1,:)'];
    else; 
        initial = [ mustar; reshape(phistar,size(phistar,1)^2,1)];
    end;% now search over delta0 and delta1 too 
    disp(minobj(initial));
  
    finalpars = fminunc(@minobj,initial,options);
    disp('after minsearch objective')
    disp(minobj(finalpars));
    % why two minimizations you ask? I found substantial variation with
    % initial conditions. I think it's because we get so close that it's
    % hard to tell how to get closer. In the final version, check
    % sensitiveity to initial conditions, by also feeding it the diagonals
    % only. 
    [ mustar phistar delta0 delta1] = decode_level(finalpars)
    
    
% ------------------------------    
% calculate A, B
% -------------------------------
    
maxcalc = 20; % how far out to calculate term structure
[A,B,Af,Bf] = affine_calculations(delta0,delta1,phistar,mustar,V,maxcalc);

% -----------------------
% new market prices of risk
% -----------------------

vec1 = zeros(14,1);
vec2 = zeros(14,1);

for i = 1:14; 
    vec1(i) = B(i,:)*V*B(i,:)';
    vec2(i) = B(i,:)*V(:,2);
end; 

lam0b = 1/2*qr'*vec1/(qr'*vec2);
lam1b = 1/(qr'*vec2); 

disp('new lambda matching only x') 
disp([lam0b lam1b]); 
disp('old lambda from cross sectional regression') 
disp([lam0a(2) lam1a(2,1)]); 


lam1 = [ [ 0 lam1b 0 0]' zeros(4,3)]; % pad out to usual matrix size
phi = phistar + V*lam1; 


% new lambda0 to nail unconditional means
%C = V(:,2); 
%lam0c = inv(C'*C)*C'*((eye(4)-phistar)*mean(X)' - mustar) - lam1b*mean(x); 

lam0c = (mean(f(:,1)) - delta0 - delta1'*inv(eye(4) - phi)*mustar)/(delta1'*inv(eye(4)-phi)*V(:,2)); 

disp('new lambda0')
disp(lam0c); 

lam0 = [ 0 lam0c 0 0]' ; 

mu = mustar + V*lam0;   

lam0l = lam0c;
lam1l = lam1b; % values for next minimization

end;

%***************************************************
% ML
% ****************************************************
% -------------------------------------------
% now, use these as starting values and do full ML
% -------------------------------------------

if 0; % switch to turn off ML
 
    initial = [ mustar; reshape(phistar,size(phistar,1)^2,1); delta0; delta1; lam0l; lam1l];
    if constr_big(initial)>=0 
        disp('******* initial value violates constraint *********'); 
    end; 
   
     fprintf('initial objective %15.7g constraint (<0?) %7.3g \n',minobj_big(initial), constr_big(initial));
    
    %    optimset fmincon shows defaults
    options = optimset('MaxFunEvals',100000,'Maxiter',20000,'Display','iter','RelLineSrchBnd',1,'RelLineSrchBndDuration',2,'TolFun',.1,'TolX',1e-3);
  
   
    middlepars = fmincon(@minobj_big,initial,[],[],[],[],[],[],@constr_big,options);
    fprintf('final objective %15.7g constraint (<0?) %7.3g \n',minobj_big(middlepars), constr_big(middlepars));
    finalpars = middlepars;
    %finalpars = fminunc(@minobj,middlepars,;
    %    disp('final objective')
    %    disp(minobj(finalpars));
    [ mustar phistar delta0 delta1 lam0l lam1l ] = decode_level_big(finalpars);
    
    lam0 = [ 0 ; lam0l; 0 ; 0; ];
    lam1 = [[0; lam1l; 0; 0] zeros(4,3)];
    
    phi = phistar + V*lam1; 
    mu = mustar + V*lam0; 
    
    % ------------------------------    
    % calculate A, B
    % -------------------------------
    
    maxcalc = 20; % how far out to calculate term structure
    %delta0 = a(1);
    %delta1 = b(1,:)'; 
    [A,B,Af,Bf] = affine_calculations(delta0,delta1,phistar,mustar,V,maxcalc);
    % now forwards f_t = Af + Bf*X_t
    % p(n)_t = A(n) + B(n,:)*X_t. B(n,:) is B_n' in paper notation
    
end; 

% ----------------------
% display everything 
% --------------------- 

disp('Factor means') 
fprintf(' data         %8.4f %8.4f %8.4f %8.4f\n', mean(X)); 
fprintf('risk neutral %8.4f %8.4f %8.4f %8.4f \n', inv(eye(4)-phistar)*mustar); 
fprintf('real         %8.4f %8.4f %8.4f %8.4f \n', inv(eye(4)-phi)*mu); % X' = mu + phi*X -- EX = (I-phi)*mu

   
 % decode results
            
disp('ols, first stage and final mustar'); 
disp(mustar_ols'); 
disp(mustar_1');
disp(mustar'); 
disp('ols first stage and actual phistar'); 
disp(phistar_ols); 
disp(phistar_1); 
disp(phistar); 
    
    
%characterize altered dynamics in table -- IR response function is below
  
errs_ols = X(1+12:T,:)- ones(T-12,1)*mu_ols' - X(1:T-12,:)*phi_ols'; 
s2 = sum(errs_ols.^2)'/T; 
mse_ols = s2.^0.5; 
R2_ols = 1-s2./var(X(1+12:T,:))'; 
        
errs_fit = X(1+12:T,:)- ones(T-12,1)*mu' - X(1:T-12,:)*phi'; 
s2 = sum(errs_fit.^2)'/T; 
mse = s2.^0.5; 
R2 = 1-s2./var(X(1+12:T,:))'; 
    
disp(' '); 
fprintf('OLS mu and phi at annual horizon\n'); 
fprintf('         %8s %8s %8s %8s %8s %8s %8s \n','const','x','level','slope','curve','mse','R2'); 
fprintf(' x       %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',100*mu_ols(1),phi_ols(1,:),100*mse_ols(1),R2_ols(1)); 
fprintf(' level   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',100*mu_ols(2),phi_ols(2,:),100*mse_ols(2),R2_ols(2)); 
fprintf(' slope   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',100*mu_ols(3),phi_ols(3,:),100*mse_ols(3),R2_ols(3)); 
fprintf(' curve   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f \n',100*mu_ols(4),phi_ols(4,:),100*mse_ols(4),R2_ols(4)); 
fprintf('eigenvalues of phi\n')
disp(eig(phi_ols)');

    % x_t+1 = mu + phi x_t + e_t+1
    % Ex = mu + phi*Ex 
    % Ex = (I-phi)^-1*mu
    disp('100*mean of the factors implied = (I-phi)^-1*mu ') 
    fprintf(' data    %8.2s %8.2f %8.2f %8.2f %8.2f \n',' ', 100*mean(X))
    fprintf('     OLS %8.2s %8.2f %8.2f %8.2f %8.2f \n',' ', 100*inv(eye(4)-phi_ols)*mu_ols)
    fprintf('   model %8.2s %8.2f %8.2f %8.2f %8.2f \n',' ', 100*inv(eye(4)-phi)*mu)


disp(' '); 
fprintf('fitted mu and phi at annual horizon\n'); 
fprintf('         %8s %8s %8s %8s %8s %8s %8s \n','const','x','level','slope','curve','mse','R2'); 
fprintf(' x       %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',100*mu(1),phi(1,:),100*mse(1),R2(1)); 
fprintf(' level   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',100*mu(2),phi(2,:),100*mse(2),R2(2)); 
fprintf(' slope   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',100*mu(3),phi(3,:),100*mse(3),R2(3)); 
fprintf(' curve   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',100*mu(4),phi(4,:),100*mse(4),R2(4)); 
fprintf('eigenvalues of fitted phi\n')
disp(eig(phi)');



% --------------------------------
% compare forward rate loadings 
% ----------------------------------

    figure; 
    subplot(2,2,1)
    plot((1:maxcalc)',Bf(1:maxcalc,1),'-vr','LineWidth',2)
    hold on
    plot(b(:,1),'ok','LineWidth',2)
    plot((1:maxcalc)',Bf_1(1:maxcalc,1),'-b','LineWidth',2)
    %plot((1:maxcalc)',Bf_ols(1:maxcalc,1),'-sb','LineWidth',2)
    %legend('Affine model, closest','f on factor regression','Affine model, ols \phi',4);
    legend('Affine model','f on factor regression',4);
    plot((0:20)',zeros(21,1),':k');
    title('return-forecast');
    %set(gca,'Xtick',1:maxcalc); 
    
    subplot(2,2,2)
    plot((1:maxcalc)',Bf(1:maxcalc,2),'-vr','LineWidth',2)
    hold on
    plot(b(:,2),'ok','LineWidth',2)
    plot((1:maxcalc)',Bf_1(1:maxcalc,2),'-b','LineWidth',2)
    %plot((1:maxcalc)',Bf_ols(1:maxcalc,2),'-sb','LineWidth',2)
    %legend('Affine model, closest','f on factor regression','Affine model, ols \phi',4);
    plot((0:20)',zeros(21,1),':k');
    title('level');
    if ~nolev
        axis([0 20 0 0.35])
    end
    if nolev
        axis([0 20 0 1.2]); 
    end
    %set(gca,'Xtick',1:maxcalc); 

    subplot(2,2,3)
    plot((1:maxcalc)',Bf(1:maxcalc,3),'-vr','LineWidth',2)
    hold on
    plot(b(:,3),'ok','LineWidth',2)
    plot((1:maxcalc)',Bf_1(1:maxcalc,3),'-b','LineWidth',2)

    %plot((1:maxcalc)',Bf_ols(1:maxcalc,3),'-sb','LineWidth',2)
    %legend('Affine model, closest','f on factor regression','Affine model, ols \phi',4);
    plot((0:20)',zeros(21,1),':k');
    title('slope');
    %set(gca,'Xtick',1:maxcalc); 

        subplot(2,2,4)
        plot((1:maxcalc)',Bf(1:maxcalc,4),'-vr','LineWidth',2)
        hold on
        plot(b(:,4),'ok','LineWidth',2)
        plot((1:maxcalc)',Bf_1(1:maxcalc,4),'-b','LineWidth',2)
    %  plot((1:maxcalc)',Bf_ols(1:maxcalc,4),'-sb','LineWidth',2)
        %legend('Affine model, closest','f on factor regression','Affine model, ols \phi',4);
        plot((0:20)',zeros(21,1),':k');
        title('curve');
        %set(gca,'Xtick',1:maxcalc); 
    
    if ~nolev
        print -depsc2 model_loadings_fed_f4.eps
    elseif nolev
        print -depsc2 model_loadings_nolev_fed.eps
    end;


% -----------------------------
% plot impulse response including altered VARs to fit forwards better
% -----------------------------

% Plot impulse-response function of factor VAR from fitted version -- how far from OLS are we, anyway

plot_ir(phistar,'Risk-Neutral'); 
  if nolev;
    print -depsc2 ir_riskneutral_nolev_fed.eps;
  end;
  if ~nolev;
      print -depsc2 ir_riskneutral_fed.eps;
  end;

plot_ir(phi_ols,'OLS') ;
  if nolev;
    print -depsc2 ir_ols_nolev_fed.eps;
  end;
  if ~nolev;
      print -depsc2 ir_ols_fed.eps;
  end;

plot_ir(phi,'ML')
  if nolev;
    print -depsc2 ir_nolev_fed_fitted.eps;
  end;
  if ~nolev;
    print -depsc2 ir_fed_fitted.eps;
  end;


% compare expected future one year rates and changes in current forward rates in response to shocks 

plot_ir_eyandf(phistar,delta1,Bf,'Risk-Neutral'); 
print -depsc2 ir_y_rn.eps;

plot_ir_eyandf(phi_ols,delta1,Bf,'OLS') ;
print -depsc2 ir_y_OLS.eps;

plot_ir_eyandf(phi,delta1,Bf,'ML')
print -depsc2 ir_y_ML.eps;
  

  
% ------------------
% plot actual, model-implied forward rates
% -------------

ffit = (ones(T,1)*Af' + X*Bf');
ffit_ols = (ones(T,1)*Af_ols' + X*Bf_ols');
ffit_1 = (ones(T,1)*Af_1' + X*Bf_1');
% --------- 
% display (plot?) fit of model
% -----------------

disp(' ' ); 
disp('***** fit of affine model.  mse of ML, 1st, ols, stat, them mad of all theses'); 
disp([...
10000*mean(mean( (ffit(:,1:15)-f(:,1:15)).^2)'     )^0.5...
10000*mean(mean( (ffit_1(:,1:15)-f(:,1:15)).^2)'     )^0.5...
10000*mean(mean( (ffit_ols(:,1:15)-f(:,1:15)).^2)' )^0.5...
10000*mean(mean( (ffit_stat(:,1:15)-f(:,1:15)).^2)')^0.5...
10000*mean(mean(abs(ffit(:,1:15)-f(:,1:15)))')...
10000*mean(mean(abs(ffit_1(:,1:15)-f(:,1:15)))')...
10000*mean(mean(abs(ffit_ols(:,1:15)-f(:,1:15)))')...
10000*mean(mean(abs(ffit_stat(:,1:15)-f(:,1:15)))')]);

disp('***** fit - stat model -- ml, 1st, ols, stat'); 
disp([...
10000*mean(mean( (ffit(:,1:15)-ffit_stat(:,1:15)).^2)'     )^0.5...
10000*mean(mean( (ffit_1(:,1:15)-ffit_stat(:,1:15)).^2)'     )^0.5...
10000*mean(mean( (ffit_ols(:,1:15)-ffit_stat(:,1:15)).^2)' )^0.5...
10000*mean(mean( (ffit_stat(:,1:15)-ffit_stat(:,1:15)).^2)')^0.5...
10000*mean(mean(abs(ffit(:,1:15)-ffit_stat(:,1:15)))')...
10000*mean(mean(abs(ffit_1(:,1:15)-ffit_stat(:,1:15)))')...
10000*mean(mean(abs(ffit_ols(:,1:15)-ffit_stat(:,1:15)))')...
10000*mean(mean(abs(ffit_stat(:,1:15)-ffit_stat(:,1:15)))')]);


% -------------------------------------------------
% model - implied expectations of various quantities
% -------------------------------------------------

% matrices for expected returns -- these should come out right!
% erx = gamma0_fit + rx_loads*X

[gamma0_fit, rx_loads, Ey1,fwd_decomp,Ext] = affine_extras(A,B,V,lam0,lam1,mu,phi,X);
[gamma0_fit_ols, rx_loads_ols, Ey1_ols,fwd_decomp_ols,Ext_ols] = affine_extras(A_ols,B_ols,V,lam0_ols,lam1_ols,mu_ols,phi_ols,X);
[gamma0_fit_rn, rx_loads_rn, Ey1_rn,fwd_decomp_rn,Ext_rn] = affine_extras(A,B,V,0*lam0,0*lam1,mustar,phistar,X);

figure
plot((2:maxmat)',qr,'--v','LineWidth',2)
hold on
plot((2:7),rx_loads(1:6,1),'-or','LineWidth',2)
plot((2:7),rx_loads_ols(1:6,1),'-sg','LineWidth',2)
legend('Original','fitted','OLS \phi',2); 
set(gca,'Xtick',2:7)

  if ~nolev
    print -depsc2 er_loads_fed.eps
  else
    print -depsc2 er_loads_nolev_fed.eps
  end;

% ----------------------
% yield curve on specific dates
% ------------------------

figure; 
for i = 1:size(dates);
    subplot(2,2,i); 
    plot((1:15)',100*(ffit(dates(i),1:15)-ffit(dates(i),1)*ones(1,15))','-kv','LineWidth',2,'MarkerSize',2); 
    hold on;
    if ~(nolev &(i == 1))
       text(15.2,100*Ext(dates(i),15)/2','E(rx^{10})/2','Color','m','BackgroundColor','w')
    end
    %text(15.3,100*ffit(dates(i),15)','Fwds','Color','k')
    %plot((1:maxcalc)',100*f(dates(i),1:maxcalc)','-ro','LineWidth',2);
    %looks the same no need for an extra line -- make fit point elsewhere
    plot((1:15)',100*(Ey1(dates(i),1:15)-Ey1(dates(i),1)*ones(1,15))','-r^','LineWidth',2);
    text(15.4,100*(Ey1(dates(i),15)-Ey1(dates(i),1)),'ML','Color','r','BackgroundColor','w')
    plot((1:15)',100*(Ey1_ols(dates(i),1:15)-Ey1_ols(dates(i),1)*ones(1,15))','-bo','LineWidth',2);
    text(15.4,100*(Ey1_ols(dates(i),15)-Ey1_ols(dates(i),1))','OLS','Color','b','BackgroundColor','w')
    plot((1:15)',100*Ext(dates(i),1:15)/2','-m','LineWidth',2); % 10 year bond excess return
    xlabel('Maturity, years');
    ylabel('Percent');
    
    
    if nolev;
        text(15.2,100*(ffit(dates(i),15)-ffit(dates(i),1)),'Fwds','Color','k');
        if i == 1; 
            text(8,100*Ext(dates(i),8)/2+0.4,'E(rx^{10})/2','Color','m','BackgroundColor','w')
        end;
    end
    
    yrplot = floor(qdate(dates(i))/10000);
    moplot = qdate(dates(i))-10000*yrplot;
    title([  num2str(moplot,'%5i') '  '  '200' num2str(yrplot,'%1i') ] );
    
    if ~nolev;
        if i == 1; 
            axis([0 18 -1.5 6]); 
        elseif i == 2; 
            axis([0 18 -1 6]); 
        elseif i == 3 
            axis([ 0 18 -2.5 2.5]); 
        end;
    end;
    
    if nolev;
        if i == 1; 
            axis([0 18 0 5]); 
        elseif i == 2; 
            axis([0 18 0 6]); 
        elseif i == 3 
            axis([ 0 18 -1.5 1]); 
        end;
    end;
end

    subplot(2,2,4)
    
    % most important last so it comes out on top. 
       
    plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,1) + 100*Ey1(indx3:T,10),'-b','LineWidth',2)
    hold on; 
    plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,5) + 100*Ey1(indx3:T,10),'-g','LineWidth',2)
    
    if 0; %shows all maturities. Cute but too complex for paper. 
        plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,2) + 100*Ey1(indx3:T,10),'-m','LineWidth',1)
        plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,3) + 100*Ey1(indx3:T,10),'-m','LineWidth',1)
        plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,4) + 100*Ey1(indx3:T,10),'-m','LineWidth',1)
    
        plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,6) + 100*Ey1(indx3:T,10),'-m','LineWidth',1)
        plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,7) + 100*Ey1(indx3:T,10),'-m','LineWidth',1) 
        plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,8) + 100*Ey1(indx3:T,10),'-m','LineWidth',1)
    end
    if 1
     place = zeros(2,1); 
     place(:) = fwd_decomp(T,9,[1 5]);
     text(plotdate(T)*ones(2,1)+1/12,100*place + 100*Ey1(T,10)*ones(2,1),...
         num2str(([1 5])','%1.0f')); 
    %plot(plotdate(indx3:T),-100*fwd_decomp(indx3:T,9,9) + 100*ffit(indx3:T,10),'-b','LineWidth',1)
    end;
   
    plot([1;1]*plotdate(dates)',[0 ;15]*ones(1,size(dates,1)),'-k','LineWidth',2)
    
    plot(plotdate(indx3:T),100*Ey1(indx3:T,10),'-r','LineWidth',2);
    text(plotdate(T)+1/12,100*Ey1(T,10)+0.2*(1-nolev),'Ey^{(1)}','Color','r','BackgroundColor','w'); 
    
    plot(plotdate(indx3:T),100*ffit(indx3:T,10),'-k','LineWidth',2);
    text(plotdate(T)+1/12,100*ffit(T,10)-0.2*(1-nolev),'10 y fwd','Color','k','BackgroundColor','w'); 
    
    
    if ~nolev
        axis([2000 2007 3 8])
    end;

    if nolev
        axis([2000 2007 2 7])
    end;



    if ~nolev 
        print -depsc2 affine_fit_fed.eps
    elseif nolev
        print -depsc2 affine_fit_fed_nolev.eps
    end;

% -----------------
% 10 year forward rate through all time
% --------------------
figure; 
    plot(plotdate(:),100*ffit(:,10),'-k','LineWidth',2);
    text(plotdate(T)+1/12,100*ffit(T,10)-0.2*(1-nolev),'10 y fwd','Color','k','BackgroundColor','w'); 
    hold on
    plot(plotdate(:),100*Ey1(:,10),'-r','LineWidth',2);
    text(plotdate(T)+1/12,100*Ey1(T,10)+0.2*(1-nolev),'Ey^{(1)}','Color','r','BackgroundColor','w'); 
    
   if 1 % Ey1 + Er component
    plot(plotdate(:),100*fwd_decomp(:,9,1) + 100*Ey1(:,10),'-b','LineWidth',2)
    plot(plotdate(:),100*fwd_decomp(:,9,2) + 100*Ey1(:,10),'-g','LineWidth',1)
    plot(plotdate(:),100*fwd_decomp(:,9,3) + 100*Ey1(:,10),'-g','LineWidth',1)
    plot(plotdate(:),100*fwd_decomp(:,9,4) + 100*Ey1(:,10),'-g','LineWidth',1)
    plot(plotdate(:),100*fwd_decomp(:,9,5) + 100*Ey1(:,10),'-b','LineWidth',2)
    plot(plotdate(:),100*fwd_decomp(:,9,6) + 100*Ey1(:,10),'-g','LineWidth',1)
    plot(plotdate(:),100*fwd_decomp(:,9,7) + 100*Ey1(:,10),'-g','LineWidth',1) 
    plot(plotdate(:),100*fwd_decomp(:,9,8) + 100*Ey1(:,10),'-g','LineWidth',1)
    plot(plotdate(:),100*fwd_decomp(:,9,9) + 100*Ey1(:,10),'-g','LineWidth',1)
    plot(plotdate(:),100*fwd_decomp(:,9,10) + 100*Ey1(:,10),'-b','LineWidth',1)
    if ~nolev
     place = zeros(8,1); 
     place(:) = fwd_decomp(T,9,1:8);
     text(plotdate(T)*ones(8,1)+1/12,100*place + 100*Ey1(T,10)*ones(8,1),...
         num2str((1:8)','%1.0f')); 
    %plot(plotdate(:),-100*fwd_decomp(:,9,9) + 100*ffit(:,10),'-b','LineWidth',1)
    end;
   end
   
    plot([1;1]*plotdate(dates)',[0 ;15]*ones(1,size(dates,1)),'-k','LineWidth',2)
    

% -----------------------
% compare Ey1 by various methods
% ------------------------

% VAR without Ex -- like original VAR. Use this version for original y forecasting VAR

if ~nolev
    [Q,L] = eig(cov(f)); % estimate three-factor dynamics
    if ~min(diag(L) == sort(diag(L)));
        disp('**************ERROR eigenvalues are in wrong order'); 
    end;

    Z = f*Q(:,end-2:end);
    [b1y,stb,R2,R2adj,v,F] = olsgmm(y(:,1),Z,12,0);
    Ey1_var = Z*b1y; % fitted value of y(1)
    
    rhs = [ones(T-12,1) Z(1:T-12,:)];
    lhs = Z(13:T,:); 
    [b1,stb,R2,R2adj,v,F] = olsgmm(lhs,rhs,12,0);  
    EZ =  [ones(T,1) Z]*b1;
    Ey1_var = [Ey1_var  EZ*b1y];  % adds Ey(1) t+2
    
    for i = 3:maxcalc;
        EZ = [ones(T,1) EZ]*b1; 
        Ey1_var = [Ey1_var EZ*b1y]; 
    end;
end;

% ----------------
% collect expected interest rate graphs
% -----------------

mats = [1 5 10 15 20];
figure;
subplot(2,2,1)
plot(plotdate,100*Ey1(:,mats)); 
title('Ey1, fitted f')
axis([1970 2007 0 15]); 
%legend(num2str(mats','%2.0f'));

subplot(2,2,2)
plot(plotdate,100*Ey1_ols(:,mats));
title('Ey1, OLS'); 
axis([1970 2007 0 15]); 

subplot(2,2,3)
plot(plotdate,100*Ey1_var(:,mats))
title('Ey1, VAR w/o x'); 
axis([1970 2007 0 15]);


    subplot(2,2,4)
    plot(plotdate(1:2),100*Ey1(1:2,mats)); 
    title('Common Legend')
    axis([1970 2007 0 20]); 
    legend([ num2str(mats','%2.0f') ones(size(mats',1),1)*' years ahead'] );

if nolev
    print -depsc2 Ey1_history_nolev.eps
end
if ~nolev
    print -depsc2 Ey1_history.eps
end    

% same, but plot CHANGES in one-year rates

figure;
subplot(2,2,1)
plot(plotdate,100*(Ey1(:,mats)-Ey1(:,1)*ones(1,length(mats)))); 
title('Ey1, fitted f')
axis([1970 2007 -5 3 ]); 
%legend(num2str(mats','%2.0f'));

subplot(2,2,2)
plot(plotdate,100*(Ey1_ols(:,mats)-Ey1_ols(:,1)*ones(1,length(mats))));
title('Ey1, OLS'); 
axis([1970 2007 -5 3 ]); 

subplot(2,2,3)
plot(plotdate,100*(Ey1_var(:,mats)-Ey1_var(:,1)*ones(1,length(mats))))
title('Ey1, VAR w/o x'); 
axis([1970 2007 -5 3 ]);


    subplot(2,2,4)
    plot(plotdate(1:2),100*(Ey1(1:2,mats)-Ey1(1:2,1)*ones(1,length(mats)))); 
    title('Common Legend')
    axis([1970 2007 -5 3 ]); 
    legend([ num2str(mats','%2.0f') ones(size(mats',1),1)*' years ahead'] );

if nolev
    print -depsc2 Ey1_diff_history_nolev.eps
end
if ~nolev
    print -depsc2 Ey1_diff_history.eps
end    



% ----------------------
% put all Ey1 on selected dates
% ----------------------

T = size(f,1);
figure; 
for i = 1:size(dates);
    subplot(2,2,i); 
    plot((1:maxmat)',100*f(dates(i),:)','--kv','LineWidth',2); 
    hold on; 
    plot((1:maxcalc)',100*Ey1(dates(i),1:maxcalc)','-b^','LineWidth',2);
    plot((1:maxcalc)',100*Ey1_ols(dates(i),1:maxcalc)','-go','LineWidth',2);
    plot((1:maxcalc)',100*Ey1_var(dates(i),1:maxcalc)','-m*','LineWidth',2);
    % legendds moved to 4th panel 
    %if nolev
    %    legend('Forwards','Ey1, fit f','Ey1, ols', 'Ey1, VAR w/o x',2); 
    %end
    %if ~nolev
    %    legend('Forwards','Ey1, fit f','Ey1, ols', 'Ey1, VAR w/o x',4); 
    %end
    yrplot = floor(qdate(dates(i))/10000);
    moplot = qdate(dates(i))-10000*yrplot;
    title(['200' num2str(yrplot,'%1i') ' ' num2str(moplot,'%5i')]); 
    axis([1 15 0 8]); 
end

    subplot(2,2,4)
    subplot(2,2,4)
    plot((1)',100*f(dates(1),1)','--kv','LineWidth',2); 
    hold on; 
    plot((1)',100*Ey1(dates(1),1)','-b^','LineWidth',2);
    plot((1)',100*Ey1_ols(dates(1),1)','-go','LineWidth',2);
    plot((1)',100*Ey1_var(dates(1),1)','-m*','LineWidth',2);
    title('Common legend')
    legend('Forwards','Ey1, fit f','Ey1, ols', 'Ey1, VAR w/o x',2) 
    set(gca,'Ytick',[])


if ~nolev
    print -depsc2 collect_Ey1.eps
end
if nolev
    print -depsc2 collect_Ey1_nolev.eps
end

% ---------------------------------
% collect recent history graphs
% --------------------------------


figure
subplot(2,2,1); 
    plot(plotdate(indx3:T),(X(indx3:T,:)-ones(T-indx3+1,1)*mean(X))./(ones(T-indx3+1,1)*std(X)),'LineWidth',2,'MarkerSize',3);
    title('Factors'); 
    legend('x','level','slope','curve'); 
    axis([2000 2006.4 -inf inf])
    
subplot(2,2,2)
    plot(plotdate(indx3:T),100*f(indx3:T,10),'-k','LineWidth',2,'MarkerSize',3); 
    hold on
   % plot(plotdate(indx3:T),100*ffit(indx3:T,10),'-kv','LineWidth',2,'MarkerSize',3);
   %        -----not needed --they're almost the same
    plot(plotdate(indx3:T),100*Ey1(indx3:T,10),'-g','LineWidth',2); 
    plot(plotdate(indx3:T),100*Ey1(indx3:T,10)+100*fwd_decomp(indx3:T,9,1),'-ro','LineWidth',2,'MarkerSize',3); 
    plot(plotdate(indx3:T),100*Ey1(indx3:T,10)+100*fwd_decomp(indx3:T,9,5),'-b','LineWidth',2);

    % this puts text on individual graphs. Instead, below, I put legends in
    % a panel
    
    %text(2006.4*ones(4,1),[100*f(T,10);100*Ey1(T,10);100*Ey1(T,10)+100*fwd_decomp(T,9,1);...
   %                       100*Ey1(T,10)+100*fwd_decomp(T,9,5)],...
    %                       {'10-yr fwd';'Ey';'Ey + 1yr rp';'Ey + 5 yr rp'})
    %axis([2000 2007.4 -inf inf])
    title('Affine, fit to forwards');
    axis([2000 2006.4 -inf inf])
    
    
subplot(2,2,3);     
    plot(plotdate(indx3:T),100*f(indx3:T,10),'-k','LineWidth',2,'MarkerSize',3); 
    hold on
    plot(plotdate(indx3:T),100*ffit_ols(indx3:T,10),'-kv','LineWidth',2,'MarkerSize',3); 
    plot(plotdate(indx3:T),100*Ey1_ols(indx3:T,10),'-g','LineWidth',2); 
    
    plot(plotdate(indx3:T),100*Ey1_ols(indx3:T,10)+100*fwd_decomp_ols(indx3:T,9,1),'-ro','LineWidth',2,'MarkerSize',3); 
    plot(plotdate(indx3:T),100*Ey1_ols(indx3:T,10)+100*fwd_decomp_ols(indx3:T,9,5),'-b','LineWidth',2);
    
    %text(2006.4*ones(5,1),[100*f(T,10);100*ffit_ols(T,10);100*Ey1_ols(T,10);100*Ey1_ols(T,10)+100*fwd_decomp_ols(T,9,1);...
     %                      100*Ey1_ols(T,10)+100*fwd_decomp_ols(T,9,5)],...
     %                      {'10-yr fwd';'fit 10-yr fwd';'Ey';'Ey + 1yr rp';'Ey + 5 yr rp'})
     %   axis([2000 2007.4 4 8])
     axis([2000 2006.4 -inf inf])
    title('Affine, OLS \phi'); 
    
    % use last panel for legends
    subplot(2,2,4)
    plot(plotdate(indx3),100*f(indx3,10),'-k','LineWidth',2,'MarkerSize',3); 
    hold on
    plot(plotdate(indx3),100*ffit_ols(indx3,10),'-kv','LineWidth',2,'MarkerSize',3); 
    plot(plotdate(indx3),100*Ey1_ols(indx3,10),'-g','LineWidth',2); 
    
    plot(plotdate(indx3),100*Ey1_ols(indx3,10)+100*fwd_decomp_ols(indx3,9,1),'-ro','LineWidth',2,'MarkerSize',3); 
    plot(plotdate(indx3),100*Ey1_ols(indx3,10)+100*fwd_decomp_ols(indx3,9,5),'-b','LineWidth',2);
    
    %text(2006.4*ones(5,1),[100*f(T,10);100*ffit_ols(T,10);100*Ey1_ols(T,10);100*Ey1_ols(T,10)+100*fwd_decomp_ols(T,9,1);...
     %                      100*Ey1_ols(T,10)+100*fwd_decomp_ols(T,9,5)],...
     %                      {'10-yr fwd';'fit 10-yr fwd';'Ey';'Ey + 1yr rp';'Ey + 5 yr rp'})
     %   axis([2000 2007.4 4 8])
     axis([2000 2006.4 4 8])
     legend( '10-yr fwd','fit 10-yr fwd','Ey','Ey + 1yr rp','Ey + 5 yr rp')
     title('Common legend') 


if nolev;
    print -depsc2 recent_history_collection_nolev.eps
end

if ~nolev;
    print -depsc2 recent_history_collection.eps
end
