% minobj objective for minimization
function ans = minobj(minpars,V,X,f,wts,min_options,which,subs)

% decode vector of parameters
% which = which parameters are contained in minpars. Ones and zeros. 
% subs = values of parameters to use if which has a zero in that place.
% subs should be full sized, same size as which and full size minpars 

restrict_eigenvalues = min_options(1); 
demean = min_options(2); 

[ mustar phistar delta0 delta1 ] = decode_level(minpars,which,subs,demean);
    
if demean
    mustar = -V(:,2)*mustar; % mustar was really lambda0l
end;

maxcalc = size(f,2);
[A,B,Af,Bf] = affine_calculations(delta0,delta1,phistar,mustar,V,maxcalc);

% Explicitly minimizes sum of squared errors -- produces same answer

ffit = ones(size(f,1),1)*Af' + X*Bf';
 if wts == 1; 
     ans = sum(mean((ffit-f).^2)');
 elseif size(wts,1) == maxcalc; 
     ans = 0; 
     for t = 1:size(f,1); 
         ans = ans + (ffit(t,:)-f(t,:))*wts*(ffit(t,:)-f(t,:))';
     end; 
     ans = ans/size(f,1);
 else; 
     disp('error in wts matrix')
 end; 



%if optim_method == 'loadings' ; 
%    % this version minimizes the weighted difference 
%    % between OLS and Affine model loadings. 
%    % it should be the fastest method. 
%    % in the end though I found the direct method was fast enough, and
%    % easier to interpret and extend to ML and weighting. 
%    if wts == 1;         
%        ans =  (Af(1) -a(1))^2 + 2*(Af(1)-a(1))*(Bf(1,:)-b(1,:))*EX + (Bf(1,:)-b(1,:))*EXX*(Bf(1,:)-b(1,:))'; 
%        for i = 2:maxcalc
%            ans = ans + (Af(i) -a(i))^2 + 2*(Af(i)-a(i))*(Bf(i,:)-b(i,:))*EX + (Bf(i,:)-b(i,:))*EXX*(Bf(i,:)-b(i,:))';
%        end;
%    elseif size(wts,1) == maxcalc;
%        for i = 1:maxcalc
%            for j = 1:maxcalc;
%                obj(i,j) = (Af(i)-a(i))*(Af(j)-a(j))...
%                    + (Af(i)-a(i))*(Bf(j,:)-b(j,:))*EX + (Af(j)-a(j))*(Bf(i,:)-b(i,:))*EX ...
%                    + (Bf(i,:)-b(i,:))*EXX*(Bf(j,:)-b(j,:))';
%            end;
%        end;
%        ans = sum(sum(wts.*obj));                
%    else; 
%        disp('errors in wts matrix'); 
%    end; 
%end;

%if optim_method == 'fit_fwds';  
%    % this version explicitly minimizes sum of squared errors relative to OLS fit
%    % thus avoids the regression error and should be a bit faster
%    % again, since the simple "forwards" was working and gave the same
%    % answer I dropped this
%    ffit = ones(size(f,1),1)*Af' + X*Bf';
%    ffit_stat = [ones(size(f,1),1) X]*[a';b'];
%    err = ffit-ffit_stat;
%    if wts == 1;  
%        ans = sum(mean(err.^2)');
%  
%        % this is an experiment -- quasi first difference to mimic weights.
%        % it works pretty well
%        %  ans = mean(err(:,1).^2) + sum(mean( (err(:,2:end-1)-.7*err(:,3:end)).^2));
%        
%    elseif size(wts,1) == maxcalc; 
%        ans = 0; 
%        for t = 1:size(f,1); 
%            ans = ans + err(t,:)*wts*err(t,:)';
%        end; 
%        ans = ans/size(f,1);
%    else; 
%        disp('error in wts matrix')
%    end; 
%end;

if restrict_eigenvalues
    ans = ans*(1+100*max(max(abs(eig(phistar)))-0.999,0)); % penalty for eigenvalues greater than one
    % it's important that this one is slightly more stringent than the
    % constraint imposed in ML tofollow, or initial value to ML will
    % violate the ML constraint
end;


