% minobj_big objective for minimization -- picks market price of risk too
function ans = minobj(minpars)
global a b wts EXX EX V X f optim_method nolev 

% passing parameters to minobj. 

% expects a vector with  mu(vector), phistar(vector), deltas, and the two
% market prices of risk lambda0l and lambda1l

% this implments a maximum likelihood estimator
% In this case the variance is fixed, and search only over the mean - min
% x'sig^-1x with sig fixed 

% NOTE: V is the innovation covariance matrix X = mu + phiX + e, 
% V =  cov(ee'). V is FIXED here, -- A,B need a V, and we use the V from the last call. 
% V updates on each call. Thus, each call
% takes the V that applied at the last call. Since the program iterates, we
% iterate to the right V

% decode vector of parameters

[ mustar phistar delta0 delta1 lam0l lam1l] = decode_level_big(minpars);
%fprintf( ' mu phi %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f \n', mustar,eig(phistar));
%fprintf( ' d  lam %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f \n',delta0, delta1,lam0l, lam1l);

maxeig = max(abs(eig(phistar))); 
maxeigphi = NaN;
flag = 0; 
if  maxeig > 0.9999;% can limit eigevnalues of phistar, but maybe we don't have to 
    ans = 1e7*max(maxeig-0.9999,0); % penalty for eigenvalues greater than one
    flag = 1; 
else; 
    maxcalc = size(f,2); % match all passed forward rates 
    num_factors = size(mustar,1); 
    [A,B,Af,Bf] = affine_calculations(delta0,delta1,phistar,mustar,V,maxcalc); % note using last V here

    ffit = ones(size(f,1),1)*Af' + X*Bf'; % model predicted forward rates
    err = ffit-f; % model errors
    
    % since the "data" are a 6 factor model, the error covariance matrix is
    % singluar. we only keep the largest eigenvalues. 
    coverr = cov(err);
    [ Q D ] = eig(coverr);
    num_eig = 4; % number of eigenvalues to keep
    Qe = err*Q(:,end-num_eig+1:end);
    D1 = inv(D(end-num_eig+1:end,end-num_eig+1:end));
    ans = 0; 
    for t = 1:size(f,1); 
        ans = ans + Qe(t,:)*D1*Qe(t,:)';
    end; 
    
    ans = ans + size(f,1)*2*pi*log(prod(diag(D(end-num_eig+1:end,end-num_eig+1:end)))); % 2 pi det sigma term
    
   % on to the true-measure dynamics
   
    T = size(f,1);
    mu = mustar + V*[0 ; lam0l; 0 ; 0]; 
    phi = phistar + V*[[0 ; lam1l; 0 ; 0] zeros(4,3)]; % Again, we're using the last V here

    maxeigphi = max(abs(eig(phi))); 
    if  maxeigphi > 0.9999; 
            ans = 1e7*max(maxeigphi-0.999,0); % penalty for eigenvalues greater than one
            flag = 1; 
    else;
        tserrs = X(13:T,:) - ones(T-12,1)*mu' - X(1:T-12,:)*phi';
        % NOW we update V -- V is global so is reset everywhere
        V = cov(tserrs); 

        % 1. likelihood of first data point
        
        % Vu = V + phi*V*phi' + phi^2*V*phi^2' +...
        % doubling algorithm, 
        % Vu = V
        % Vu = V + phiV*phi'
        % Vu = (V + phiV*phi')+ phi^2(V + phiV*phi')*(phi^2)'
        if maxeigphi < 1; 
                Vu = V; 
            for j = 1:10; 
                Vu = Vu + (phi^j)*Vu*(phi^j)';
            end; 
    
            ans = ans + (X(1,:)'- (inv(eye(4))-phi)*mu)'*inv(Vu)*(X(1,:)'- (inv(eye(4))-phi)*mu);
            ans = ans + 2*pi*det(Vu); 
        end;    
        
        % liikelyhood of remaining data points
        
        % approach 1: find implied monthly model
        phimo = real(phi^(1/12)); % real truncates imaginary terms that come from numerical approximation
        % mu = mumo*(I +phimo + phimo^2 + ...+phimo^12)*mumo 
        % mu = (I-phimo)^-1*(I-phimo^13)*mumo
        % (I-phimo)*(I-phimo^13)^-1*mu = mumo
        mumo = (eye(4)-phimo)*inv(eye(4)-phimo^13)*mu;
        tserrmo = X(2:T,:) - ones(T-1,1)*mumo' - X(1:T-1,:)*phimo';
        Vmo = cov(tserrmo);    
        Vmo1 = inv(Vmo);
        for t = 1:T-1; 
            ans = ans + tserrmo(t,:)*Vmo1*tserrmo(t,:)';
        end; 
        ans = ans + (T-1)*2*pi*log(det(Vmo)); 
        

        % approach 2: 
        % under the null, tserrs have correlation V contemporaneously and an MA(12)
        % temporal structure. Approximate as an AR(1) with correlation coefficient
        % 10/12 = rho, common to all elements
        % e_t = rho I e_t-1 + dt
        % var(et) = var(d) + (rho var(d) rho' + rho^2 var(d) reho2' + ... 
        %= (1+rho^2 + rho^4...) var(d) = 1/(1-rho^2) var(d)
        % so var(d) = (1-rho^2) var(e)
        % L = err(1)'V^-1*err(1)
        if 0; 
            rho = 10/12;
            ans = ans+tserrs(1,:)*inv(V)*tserrs(1,:)'; % probability of first observation
            for t = 14:T; 
                ans = ans+(1-rho^2)*(tserrs(t-12,:)-rho*tserrs(t-12,:))*inv(V)*(tserrs(t-12,:)-rho*tserrs(t-12,:))';
            end;
        end; 
        
        
    end;
end;

%fprintf('objective %9.3f eig(phistar) %9.3f eig(phi) %9.3f violating? %3.0f \r ',ans, maxeig, maxeigphi, flag)



