% show_results
% show results of a generic estimate
% this allows you to experiment -- final graphs which compare different
% versions in the main program 

function [A,B,Af,Bf,mu,phi,lam0,lam1,gamma0_fit,rx_loads,Ey1,fwd_decomp,Ext] = show_results(delta0,delta1,lambdas,phistar,mustar,X,V,f,f_ma_fb,graph_label,dates,qdate,plotdate,indx3)

disp('*****************')
fprintf('Results for case = %12.12s\n',graph_label)
disp(' ');

maxcalc = 20; % how far out to calculate term structure
[A,B,Af,Bf] = affine_calculations(delta0,delta1,phistar,mustar,V,maxcalc);

lam0 = [ 0 lambdas(1) 0 0]'; 
lam1 = [ [ 0 lambdas(2) 0 0]' zeros(4,3)]; % pad out to usual matrix size
phi = phistar + V*lam1;
mu = mustar + V*lam0;    

T = size(f,1);
ffit = (ones(T,1)*Af' + X*Bf');

rhs = [ones(T,1) X];
[b1,stb,R2,R2adj,v,F] = olsgmm(f,rhs,0,0);
a = b1(1,:)';
b = b1(2:end,:)';
ffit_stat = rhs*b1; 

% --------- 
% display (plot?) fit of model
% -----------------

disp(' ' ); 
disp('***** fit of affine model.  mse of estimated and statistical, stat, them mad of all theses'); 
disp([...
10000*mean(mean( (ffit(:,1:15)-f(:,1:15)).^2)'     )^0.5...
10000*mean(mean( (ffit_stat(:,1:15)-f(:,1:15)).^2)')^0.5...
10000*mean(mean(abs(ffit(:,1:15)-f(:,1:15)))')...
10000*mean(mean(abs(ffit_stat(:,1:15)-f(:,1:15)))')]);

disp('***** fit - stat model -- estimated, then stat'); 
disp([...
10000*mean(mean( (ffit(:,1:15)-ffit_stat(:,1:15)).^2)'     )^0.5...
10000*mean(mean( (ffit_stat(:,1:15)-ffit_stat(:,1:15)).^2)')^0.5...
10000*mean(mean(abs(ffit(:,1:15)-ffit_stat(:,1:15)))')...
10000*mean(mean(abs(ffit_stat(:,1:15)-ffit_stat(:,1:15)))')]);


% ----------------------
% display everything 
% --------------------- 

disp('Factor means') 
fprintf(' data         %8.4f %8.4f %8.4f %8.4f\n', mean(X)); 
fprintf('model         %8.4f %8.4f %8.4f %8.4f \n', inv(eye(4)-phi)*mu); % X' = mu + phi*X -- EX = (I-phi)*mu

%characterize altered dynamics in table -- IR response function is below

errs_fit = X(1+12:T,:)- ones(T-12,1)*mu' - X(1:T-12,:)*phi'; 
s2 = sum(errs_fit.^2)'/T; 
mse = s2.^0.5; 
R2 = 1-s2./var(X(1+12:T,:))'; 
  
disp(' '); 
fprintf('fitted mu and phi at annual horizon\n'); 
fprintf('         %8s %8s %8s %8s %8s %8s %8s \n','100*mu','x','level','slope','curve','mse','R2'); 
fprintf(' x       %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',100*mu(1),phi(1,:),100*mse(1),R2(1)); 
fprintf(' level   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',100*mu(2),phi(2,:),100*mse(2),R2(2)); 
fprintf(' slope   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',100*mu(3),phi(3,:),100*mse(3),R2(3)); 
fprintf(' curve   %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n',100*mu(4),phi(4,:),100*mse(4),R2(4)); 
disp(' ');
fprintf('abs values of eigenvalues\n')
fprintf('phi:     %8.2f %8.2f %8.2f %8.2f \n', abs(eig(phi)));


% --------------------------------
% compare forward rate loadings 
% ----------------------------------

figure;
subplot(2,2,1)
plot((1:maxcalc)',Bf(1:maxcalc,1),'-b','LineWidth',2)
hold on
plot(b(:,1),'ok','LineWidth',2)
legend([graph_label ' model'],'f on X regression',4);
plot((0:20)',zeros(21,1),':k');
title('Loading B^f on x');
xlabel('Maturity')

subplot(2,2,2)
plot((1:maxcalc)',Bf(1:maxcalc,2),'-b','LineWidth',2)
hold on
plot(b(:,2),'ok','LineWidth',2)
plot((0:20)',zeros(21,1),':k');
title('Level');
xlabel('Maturity')

subplot(2,2,3)
plot((1:maxcalc)',Bf(1:maxcalc,3),'-b','LineWidth',2)
hold on
plot(b(:,3),'ok','LineWidth',2)
plot((0:20)',zeros(21,1),':k');
title('Slope');
xlabel('Maturity')

subplot(2,2,4)
plot((1:maxcalc)',Bf(1:maxcalc,4),'-b','LineWidth',2)
hold on
plot(b(:,4),'ok','LineWidth',2)
plot((0:20)',zeros(21,1),':k');
title('curve');
xlabel('Maturity')

  print('-depsc2', [graph_label '_loadings.eps']);

% -----------------------------
% plot impulse response including altered VARs to fit forwards better
% -----------------------------

% Plot impulse-response function of factor VAR from fitted version -- how far from OLS are we, anyway

plot_ir(phi,graph_label)
 print('-depsc2', [graph_label '_ir.eps']);


% compare expected future one year rates and changes in current forward rates in response to shocks 

plot_ir_eyandf(phi,delta1,Bf,graph_label)
print('-depsc2', [graph_label '_irEy.eps']);

% -------------------------------------------------
% model - implied expectations of various quantities
% -------------------------------------------------

% matrices for expected returns -- these should come out right!
% erx = gamma0_fit + rx_loads*X

[gamma0_fit, rx_loads, Ey1, fwd_decomp, Ext] = affine_extras(A,B,V,lam0,lam1,mu,phi,X);

% ----------------------
% yield curve on specific dates
% ------------------------

if sum(dates)>0; % use dates = -99 with subsamples to turn this off
    figure;
    for i = 1:size(dates);
        subplot(2,2,i);
        plot((1:15)',100*ffit(dates(i),1:15)','-bv','LineWidth',2,'MarkerSize',2);
        hold on; 
        % using ffit looks like cheating, but it is only 10 bp different. 
        % If not, you have a 10 bp difference between ffit and Ey which looks eair and takes too long to explain
        plot((1:5)',100*f_ma_fb(dates(i),:)','-ks','linewidth',2,'MarkerSize',2);
        plot((1:15)',100*Ey1(dates(i),1:15)','-r^','LineWidth',2);
        plot((1:15)',100*Ext(dates(i),1:15)/2','-m','LineWidth',2); 
    % 10  year bond excess return. 
    ylabel('Percent');

    if i == 1; 
            text(5.1,100*f_ma_fb(dates(i),5),'FB fwd','Color','k','BackgroundColor','w')
    end;
    if (i == 1)|(i == 2);; 
       text(10,100*Ext(dates(i),10)/2','E(rx^{(10)})/2','Color','m','BackgroundColor','w')
       text(10,100*ffit(dates(i),10),'Fwds','Color','b','BackgroundColor','w');
       text(10,100*Ey1(dates(i),10),'Ey^{(1)}','Color','r','BackgroundColor','w')
    end;
       %    if i == 1;
        %        text(8,100*Ext(dates(i),8)/2+0.4,'E(rx^{10})/2','Color','m','BackgroundColor','w')
        %    end;

        yrplot = floor(qdate(dates(i))/10000);
        moplot = qdate(dates(i))-10000*yrplot;
        title([  num2str(moplot,'%5i') '  '  '200' num2str(yrplot,'%1i') ] );

        %if sum(graph_label) == sum('Return-forecast')
            axis([0 15 -2 7]);
        %end;
        
    end

    subplot(2,2,4)

    % most important last so it comes out on top.

    %plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,1) + 100*Ey1(indx3:T,10),'-b','LineWidth',2)
    %hold on;
    plot(plotdate(indx3:T),100*fwd_decomp(indx3:T,9,5) + 100*Ey1(indx3:T,10),'-g','LineWidth',2)
    hold on
    title('10 yr fwd history');
    if 1
        place = zeros(2,1);
        place(:) = fwd_decomp(T,9,[1 5]);
        text(plotdate(dates(1)+2),5.2,'5','Color','g');
    end;

    plot([1;1]*plotdate(dates)',[0 ;15]*ones(1,size(dates,1)),'-k','LineWidth',2); % zero line
    plot(plotdate(indx3:T),100*Ey1(indx3:T,10),'-r^','Markersize',2,'LineWidth',2); % expected yield 10 years out
    plot(plotdate(indx3:T),100*ffit(indx3:T,10),'-bv','Markersize',2,'LineWidth',2);
    plot(plotdate(indx3:T),100*Ey1(indx3:T,1),'-k'); % current interest rate

    text(2002.4,100*Ey1(dates(1)+2,10),'Ey^{(1)}_{t+9}','Color','r');
    text(plotdate(dates(1)+2),7.5,'f^{(10)}','Color','b');
    text(2004.7,1.5,'y^{(1)}_t','Color','k')
    
    %if sum(graph_label) == sum('Return-forecast')
        axis([2000 2006.5 0 8]);
    %end;
    print('-depsc2', [graph_label '_recent_hist.eps']);
end;

% -----------------
% 5 and 10 year forward rate through all time with risk premia
% --------------------

figure;

plot(plotdate(:),100*fwd_decomp(:,4,1),'-g')
hold on
plot(plotdate(:),100*fwd_decomp(:,4,2),'-g')
plot(plotdate(:),100*fwd_decomp(:,4,3),'-g')
plot(plotdate(:),100*fwd_decomp(:,4,4),'-g')
plot(plotdate(:),100*fwd_decomp(:,4,5),'-g')
plot(plotdate(:),100*(ffit(:,5)-Ey1(:,5)),'-r')
plot(plotdate(:),zeros(T,1),'-k')
axis([1971 2007 -inf inf]); 
%legend('1st year prem','2nd year prem','3d year prem','4th year prem','5th year prem','fwd - exp. spot',4)
title(['5 year forward premium decomposition, ' graph_label ' model']);
print('-depsc2', [graph_label '_5yr_fwd_decomp.eps']);


figure
% Ey1 + Er component
plot(plotdate(:),100*fwd_decomp(:,9,1) + 100*Ey1(:,10),'-k','LineWidth',2)
hold on
plot(plotdate(:),100*fwd_decomp(:,9,2) + 100*Ey1(:,10),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,9,3) + 100*Ey1(:,10),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,9,4) + 100*Ey1(:,10),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,9,5) + 100*Ey1(:,10),'-k','LineWidth',2)
plot(plotdate(:),100*fwd_decomp(:,9,6) + 100*Ey1(:,10),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,9,7) + 100*Ey1(:,10),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,9,8) + 100*Ey1(:,10),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,9,9) + 100*Ey1(:,10),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,9,10) + 100*Ey1(:,10),'-k','LineWidth',1)

place = zeros(8,1);
place(:) = fwd_decomp(T,9,1:8);
%text(plotdate(T)*ones(8,1)+1/12,100*place + 100*Ey1(T,10)*ones(8,1),...
    %num2str((1:8)','%1.0f'));
% fwd and Ey
plot(plotdate(:),100*f(:,10),'-b','LineWidth',2);
%text(plotdate(T)+1/12,100*ffit(T,10)-0.2,'10 y fwd','Color','k','BackgroundColor','w');
hold on
plot(plotdate(:),100*Ey1(:,10),'-r','LineWidth',2);
%text(plotdate(T)+1/12,100*Ey1(T,10)+0.2,'Ey^{(1)}','Color','r','BackgroundColor','w');
%plot([1;1]*plotdate(dates)',[0;15]*ones(1,size(dates,1)),'-k','LineWidth',2)
% vertical lines to denote period of other graphs
axis([1971 2007 0 20]);
title(['10 year fwd, Ey^{(1)}, risk premium, ' graph_label]);
print('-depsc2', [graph_label '_10yrfwdhist.eps']);


% same, but 5 years
figure;

% Ey1 + Er component
plot(plotdate(:),100*fwd_decomp(:,4,1) + 100*Ey1(:,5),'-k','LineWidth',2)
hold on
plot(plotdate(:),100*fwd_decomp(:,4,2) + 100*Ey1(:,5),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,4,3) + 100*Ey1(:,5),'-g','LineWidth',1)
plot(plotdate(:),100*fwd_decomp(:,4,4) + 100*Ey1(:,5),'-g','LineWidth',1)

place = zeros(4,1);
place(:) = fwd_decomp(T,4,1:4);
%text(plotdate(T)*ones(8,1)+1/12,100*place + 100*Ey1(T,10)*ones(8,1),...
    %num2str((1:8)','%1.0f'));
% fwd and Ey
plot(plotdate(:),100*f(:,5),'-b','LineWidth',2); % f or ffit?
%text(plotdate(T)+1/12,100*ffit(T,10)-0.2,'10 y fwd','Color','k','BackgroundColor','w');
hold on
plot(plotdate(:),100*Ey1(:,5),'-r','LineWidth',2);
%text(plotdate(T)+1/12,100*Ey1(T,10)+0.2,'Ey^{(1)}','Color','r','BackgroundColor','w');
%plot([1;1]*plotdate(dates)',[0;15]*ones(1,size(dates,1)),'-k','LineWidth',2)
% vertical lines to denote period of other graphs
axis([1971 2007 0 20]);
title(['5 year fwd, Ey^{(1)}, risk premium, ' graph_label]);
print('-depsc2', [graph_label '_5yrfwdhist.eps']);


% ---------------------------------------
% term structure of expected returns
%-----------------------------------

figure
plot(plotdate(:),Ext);
hold on;
plot(plotdate(:),Ext(:,[1 6  11 16]),'LineWidth',2);
place = zeros(8,1);
place = Ext(T,[1 6 11 16]);
text((plotdate(T)+1/12)*ones(4,1),place,num2str([0 5 10 15]','%2.0f'));
title('Term structure of 10 Y expected excess returns E_t(r^{(10)}_{t+j})')
print('-depsc2', [graph_label '_Exp_ret_term_structure.eps']);

% ----------------
% Expected interest rate for full sample
% -----------------

mats = [1 2 6 11 ];
figure;
plot(plotdate,100*Ey1(:,mats),'linewidth',1.5); 
title(['Expected one-year rate Ey^{(1)}, ' graph_label ' model']); 
axis([1971 2007 0 20]); 
legend( 'Current y^{(1)}' , ' 1 year', '5 years', '10 years' );
print('-depsc2', [graph_label '_int_rate_forecast.eps']);

% ----------------------------------
% Atkeson graph of yields, for people who don't understand forward rates
% ----------------------------------

y = cumsum(f')';
y = y./(ones(T,1)*(1:size(y,2))); % recreate yields rather than add a variable to function

yeh = cumsum(Ey1')'; 
yeh = yeh./(ones(T,1)*(1:size(yeh,2))); % expectations hypothesis version

figure;
plot(plotdate,100*y(:,1),'-k'); 
hold on
plot(plotdate,100*y(:,5),'linewidth',2); 
plot(plotdate,100*yeh(:,5),'-r','linewidth',2); 
axis([1970 2007 0 20])
legend('1-Yr Yield','5-Yr Yield','Exp Hypo.')
title(['5 year yield decomposition, ' graph_label ' model']) 
print('-depsc2', [graph_label '_5yr_yield.eps']);
hgsave([graph_label '_5yr_yield.fig']);

figure;
plot(plotdate,100*y(:,1),'-k'); 
hold on
plot(plotdate,100*y(:,10),'linewidth',2); 
plot(plotdate,100*yeh(:,10),'-r','linewidth',2); 
axis([1970 2007 0 20])
legend('1-Yr Yield','10-Yr Yield','Exp Hypo.')
title(['10 year yield decomposition, ' graph_label ' model']) 
print('-depsc2', [graph_label '_10yr_yield.eps']);
hgsave([graph_label '_10yr_yield.fig']);

% similar simple graph with forward rates

figure;
plot(plotdate,100*f(:,1),'-k'); 
hold on
plot(plotdate,100*f(:,5),'linewidth',2); 
plot(plotdate,100*Ey1(:,5),'-r','linewidth',2); 
axis([1971 2007 0 20])
legend('1-Yr Yield','5-Yr fwd','Expected Y^{(1)}')
title(['5 year forward decomposition, ' graph_label ' model']) 
print('-depsc2', [graph_label '_5yr_forward.eps']);

figure;
plot(plotdate,100*f(:,1),'-k'); 
hold on
plot(plotdate,100*f(:,10),'linewidth',2); 
plot(plotdate,100*Ey1(:,10),'-r','linewidth',2); 
axis([1971 2007 0 20])
legend('1-Yr Yield','10-Yr fwd','Expected Y^{(1)}')
title(['10 year forward decomposition, ' graph_label ' model']) 
print('-depsc2', [graph_label '_10yr_forward.eps']);

% ---------------------------------
% collect recent history graphs
% --------------------------------

if sum(dates >0);
figure
    plot(plotdate(indx3:T),(X(indx3:T,:)-ones(T-indx3+1,1)*mean(X))./(ones(T-indx3+1,1)*std(X)),'LineWidth',2,'MarkerSize',3);
    title('Factors -- recent history' ); 
    legend('x','level','slope','curve'); 
    axis([2000 2006.4 -inf inf])
    
figure
    plot(plotdate(indx3:T),100*f(indx3:T,10),'-k','LineWidth',2,'MarkerSize',3); 
    hold on
    title(['Ey^{(1)}, ' graph_label  ' model']); 
   % plot(plotdate(indx3:T),100*ffit(indx3:T,10),'-kv','LineWidth',2,'MarkerSize',3);
   %        -----not needed --they're almost the same
    plot(plotdate(indx3:T),100*Ey1(indx3:T,10),'-g','LineWidth',2); 
    plot(plotdate(indx3:T),100*Ey1(indx3:T,10)+100*fwd_decomp(indx3:T,9,1),'-ro','LineWidth',2,'MarkerSize',3); 
    plot(plotdate(indx3:T),100*Ey1(indx3:T,10)+100*fwd_decomp(indx3:T,9,5),'-b','LineWidth',2);

    legend('10 yr fwd','Ey','Ey + 1yr prem.','Ey + 5yr prem',3);
    % this puts text on individual graphs. Instead, below, I put legends in
    % a panel
    
    %text(2006.4*ones(4,1),[100*f(T,10);100*Ey1(T,10);100*Ey1(T,10)+100*fwd_decomp(T,9,1);...
   %                       100*Ey1(T,10)+100*fwd_decomp(T,9,5)],...
    %                       {'10-yr fwd';'Ey';'Ey + 1yr rp';'Ey + 5 yr rp'})
    %axis([2000 2007.4 -inf inf])
    axis([2000 2006.4 -inf inf])
      print('-depsc2', [graph_label '_10yrfwd_recent.eps']);
end

    
    
